/*
 * Copyright (C) 2002, 2003 Emmanuel Saracco <esaracco@noos.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "application.h"


#define PROGRESS_LABEL_MAX_LEN 70


/* hash table of the status code
   properties (message, bg etc.) */
static GHashTable *uc_application_status_code_properties_hash;


extern gchar **environ;

static void uc_application_menu_init (void);
static void uc_application_clean_data (void);
static void uc_application_settings_init (void);
static void uc_application_status_code_properties_free_callback (gpointer key,
								 gpointer
								 value,
								 gpointer
								 user_data);
static void uc_application_buttons_display (const GtkWidget * frame,
					    gpointer array,
					    const guint items_size,
					    gpointer selected_array,
					    const guint selected_items_size);
static GList *uc_application_buttons_get_list (const GtkWidget * widget);
static void uc_application_buttons_save_setting (GtkWidget * widget,
						 gchar * path);
static gchar *uc_application_format_progress_dialog_label (const gchar *
							   label);
static void uc_application_page_information_dialog_display_links (const
								  UCLinkProperties
								  * prop);
static void uc_application_page_information_dialog_display_emails (const
								   UCLinkProperties
								   * prop);
static void uc_application_page_information_dialog_display_header (const
								   UCLinkProperties
								   * prop);
static void uc_application_page_information_dialog_display_meta_tags (const
								      UCLinkProperties
								      * prop);
static void uc_application_settings_dialog_display_filters_list (const gchar *
								 glade,
								 gpointer
								 items,
								 guint
								 items_size,
								 GtkTreeView
								 ** tv,
								 GtkListStore
								 ** model);
static void uc_application_filters_save_settings (GtkTreeView * treeview,
						  const gchar * path);
static void uc_application_status_code_properties_free (void);
static gboolean
uc_application_page_information_dialog_display_header_is_valid (const gchar *
								status_code);


/*
 * show dialog for saving or not bookmarks
 * change
 */
void
uc_application_save_bookmarks (void)
{
  if (uc_application_dialog_yes_no_show
      (_
       ("Bookmarks modifications will be saved.\n"
	"Your old bookmarks file will be renamed "
	"with \".uc\" extension.\n\n" "Do you want to save?"),
       GTK_MESSAGE_QUESTION))
    uc_bookmarks_save_changes ();
}

void
uc_application_build_url_treeview (void)
{
  GtkCellRenderer *renderer = NULL;
  GtkTreeViewColumn *column = NULL;

  gtk_tree_view_set_model (treeview, GTK_TREE_MODEL (treestore));

  /* ID column */
  renderer = gtk_cell_renderer_text_new ();

  gtk_tree_view_insert_column_with_attributes (treeview, -1,
					       _("Id"), renderer,
					       "text", UID_COLUMN, NULL);
  column = gtk_tree_view_get_column (treeview, UID_COLUMN);
  gtk_tree_view_column_set_visible (column, FALSE);

  /* LINK STATUS column */
  column = gtk_tree_view_column_new ();
  gtk_tree_view_column_set_title (column, _("Status"));
  gtk_tree_view_column_set_alignment (column, 0.5);
  renderer = gtk_cell_renderer_pixbuf_new ();
  gtk_tree_view_column_pack_start (column, renderer, FALSE);
  gtk_tree_view_column_set_attributes (column, renderer,
				       "pixbuf",
				       LINK_STATUS_ICON_COLUMN, NULL);
  gtk_tree_view_column_set_resizable (column, TRUE);
  gtk_tree_view_append_column (treeview, column);

  /* LINK TYPE column */
  renderer = gtk_cell_renderer_pixbuf_new ();
  gtk_tree_view_insert_column_with_attributes (treeview, -1,
					       _("Type"), renderer,
					       "pixbuf",
					       LINK_ICON_COLUMN, NULL);
  column = gtk_tree_view_get_column (treeview, LINK_ICON_COLUMN);
  gtk_tree_view_column_set_resizable (column, TRUE);

  /* LABEL column */
  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (treeview, -1,
					       _("Label"), renderer,
					       "text", LABEL_COLUMN, NULL);
  uc_utils_set_userfriendly_treeview_column (treeview, LABEL_COLUMN);

  /* URL column */
  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (treeview, -1,
					       _("URL"), renderer,
					       "text", URL_COLUMN, NULL);
  uc_utils_set_userfriendly_treeview_column (treeview, URL_COLUMN);

  gtk_tree_view_set_rules_hint (treeview, TRUE);

}


/*
 * display the whole list of checked url in the
 * main url treeview
 */
void
uc_application_main_tree_display_all (void)
{
  GtkTreeIter iter[UC_MAX_DEPTH_LEVEL];
  guint i_iter = 0;

  uc_application_menu_set_sensitive_all ("display", TRUE);
  gtk_tree_store_clear (treestore);
  uc_check_display_list (uc_lists_checked_links_get (), iter, i_iter, 0);
  UC_WIDGET_SET_SENSITIVE (glade_main_window, "display_all", FALSE);
  UC_WIDGET_SET_SENSITIVE (glade_main_window, "display_collapse_all", FALSE);
}


/*
 * display the whole list of bad links in the
 * main url treeview
 */
void
uc_application_main_tree_display_badlinks (void)
{
  GtkTreeIter iter;

  uc_application_menu_set_sensitive_all ("display", TRUE);
  gtk_tree_store_clear (treestore);
  uc_check_display_list_with_only (uc_lists_checked_links_get (), iter,
				   UC_CHECK_DISPLAY_BADLINKS);
  UC_WIDGET_SET_SENSITIVE (glade_main_window, "display_badlinks", FALSE);
  UC_WIDGET_SET_SENSITIVE (glade_main_window, "display_collapse_all", FALSE);
  UC_WIDGET_SET_SENSITIVE (glade_main_window, "display_expand_all", FALSE);
}


/*
 * display the whole list of good links in the
 * main url treeview
 */
void
uc_application_main_tree_display_goodlinks (void)
{
  GtkTreeIter iter;

  uc_application_menu_set_sensitive_all ("display", TRUE);
  gtk_tree_store_clear (treestore);
  uc_check_display_list_with_only (uc_lists_checked_links_get (), iter,
				   UC_CHECK_DISPLAY_GOODLINKS);
  UC_WIDGET_SET_SENSITIVE (glade_main_window, "display_goodlinks", FALSE);
  UC_WIDGET_SET_SENSITIVE (glade_main_window, "display_collapse_all", FALSE);
  UC_WIDGET_SET_SENSITIVE (glade_main_window, "display_expand_all", FALSE);
}


/*
 * display the whole list of links with timeout in the
 * main url treeview
 */
void
uc_application_main_tree_display_timeouts (void)
{
  GtkTreeIter iter;

  uc_application_menu_set_sensitive_all ("display", TRUE);
  gtk_tree_store_clear (treestore);
  uc_check_display_list_with_only (uc_lists_checked_links_get (), iter,
				   UC_CHECK_DISPLAY_TIMEOUTS);
  UC_WIDGET_SET_SENSITIVE (glade_main_window, "display_timeouts", FALSE);
  UC_WIDGET_SET_SENSITIVE (glade_main_window, "display_collapse_all", FALSE);
  UC_WIDGET_SET_SENSITIVE (glade_main_window, "display_expand_all", FALSE);
}


/*
 * display the whole list of malformed links in the
 * main url treeview
 */
void
uc_application_main_tree_display_malformedlinks (void)
{
  GtkTreeIter iter;

  uc_application_menu_set_sensitive_all ("display", TRUE);
  gtk_tree_store_clear (treestore);
  uc_check_display_list_with_only (uc_lists_checked_links_get (), iter,
				   UC_CHECK_DISPLAY_MALFORMED);
  UC_WIDGET_SET_SENSITIVE (glade_main_window,
			   "display_malformedlinks", FALSE);
  UC_WIDGET_SET_SENSITIVE (glade_main_window, "display_collapse_all", FALSE);
  UC_WIDGET_SET_SENSITIVE (glade_main_window, "display_expand_all", FALSE);
}


/*
 * expand the main url treeview
 */
void
uc_application_main_tree_expand_all (void)
{
  uc_application_menu_set_sensitive_all ("display", TRUE);
  gtk_tree_view_expand_all (treeview);
  UC_WIDGET_SET_SENSITIVE (glade_main_window, "display_expand_all", FALSE);
}


/*
 * collapse the main url treeview
 */
void
uc_application_main_tree_collapse_all (void)
{
  uc_application_menu_set_sensitive_all ("display", TRUE);
  gtk_tree_view_collapse_all (treeview);
  UC_WIDGET_SET_SENSITIVE (glade_main_window, "display_collapse_all", FALSE);
}


/*
 * display meta tags
 */
static void
uc_application_page_information_dialog_display_meta_tags (const
							  UCLinkProperties *
							  prop)
{
  enum
  {
    NAME_COLUMN,
    CONTENT_COLUMN,
    N_COLUMNS
  };
  GtkTreeView *tv;
  GtkTreeIter iter;
  GtkListStore *model;
  GtkCellRenderer *renderer;
  GtkWidget *scroll = NULL;
  GList *item = NULL;

  UC_GLADE_GET_WIDGET ("general_scrolled_down", glade_page_information_dialog,
		       scroll);

  model = gtk_list_store_new (N_COLUMNS, G_TYPE_STRING, G_TYPE_STRING);

  item = g_list_first (prop->metas);
  while (item)
    {
      UCHTMLTag *lp = (UCHTMLTag *) item->data;

      gtk_list_store_append (model, &iter);
      gtk_list_store_set (model, &iter,
			  NAME_COLUMN, (gchar *) lp->label,
			  CONTENT_COLUMN, (gchar *) lp->value, -1);

      item = g_list_next (item);
    }

  tv = (GtkTreeView *) gtk_tree_view_new_with_model ((GtkTreeModel *) model);
  g_object_unref (model);

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (tv,
					       -1,
					       _("Name"), renderer,
					       "text", NAME_COLUMN, NULL);
  uc_utils_set_userfriendly_treeview_column (tv, NAME_COLUMN);

  gtk_tree_view_insert_column_with_attributes (tv,
					       -1,
					       _("Content"), renderer,
					       "text", CONTENT_COLUMN, NULL);
  uc_utils_set_userfriendly_treeview_column (tv, CONTENT_COLUMN);

  gtk_tree_view_set_rules_hint (tv, TRUE);

  gtk_container_add (GTK_CONTAINER (scroll), GTK_WIDGET (tv));

  gtk_widget_show_all (scroll);
}


/*
 * show the search dialog
 */
void
uc_application_new_search_dialog_show (void)
{
  UC_GLADE_INIT ("new_search_dialog", glade_new_search_dialog,
		 new_search_dialog);

  UC_WIDGET_SET_SENSITIVE (glade_new_search_dialog, "search_links_name",
			   FALSE);
  UC_WIDGET_SET_SENSITIVE (glade_new_search_dialog, "search_meta_content",
			   FALSE);
  UC_WIDGET_SET_SENSITIVE (glade_new_search_dialog, "search_emails", FALSE);

  gtk_widget_show_all (new_search_dialog);
}


/*
 * show the search dialog
 */
void
uc_application_search_dialog_show (void)
{
  UC_GLADE_INIT ("search_dialog", glade_search_dialog, search_dialog);

  gtk_widget_show_all (search_dialog);
}


/*
 * show the add domain filter
 * dialog
 */
void
uc_application_add_filter_domain_dialog_show (void)
{
  UC_GLADE_INIT ("add_filter_domain_dialog", glade_add_filter_domain_dialog,
		 add_filter_domain_dialog);

  gtk_widget_show_all (add_filter_domain_dialog);
}


/*
 * show the add directory filter
 * dialog
 */
void
uc_application_add_filter_directory_dialog_show (void)
{

  UC_GLADE_INIT ("add_filter_directory_dialog",
		 glade_add_filter_directory_dialog,
		 add_filter_directory_dialog);

  gtk_widget_show_all (add_filter_directory_dialog);
}


/*
 * show the properties dialog
 */
void
uc_application_page_information_dialog_show (void)
{
  gint id = 0;
  UCLinkProperties *prop = NULL;

  if ((id = uc_check_treeview_get_selected_row_id ()) < 0)
    return;

  prop = uc_lists_checked_links_lookup_by_uid (id);

  /* do not display link information for bookmark's
   * folders */
  if (prop->link_type == LINK_TYPE_BOOKMARK_FOLDER)
    return;

  UC_GLADE_INIT ("page_information_dialog", glade_page_information_dialog,
		 page_information_dialog);

  uc_application_page_information_dialog_display_header (prop);
  uc_application_page_information_dialog_display_meta_tags (prop);
  uc_application_page_information_dialog_display_links (prop);
  uc_application_page_information_dialog_display_emails (prop);

  gtk_widget_show_all (page_information_dialog);
}


/*
 * return a formated label to display in the
 * progress dialog box
 */
static gchar *
uc_application_format_progress_dialog_label (const gchar * label)
{
  gchar *str = NULL;
  gchar *tmp = NULL;

  if (strlen (label) > (PROGRESS_LABEL_MAX_LEN - 5))
    {
      gchar *tmp_pre = NULL;
      gchar *tmp_post = NULL;
      guint cut_point = (PROGRESS_LABEL_MAX_LEN / 2);

      tmp_pre = g_memdup (label, (cut_point - 3) + 1);
      tmp_pre[cut_point - 3] = '\0';
      tmp_post =
	g_memdup (&label
		  [strlen (label) - (cut_point - 2) + 1],
		  (cut_point + 2) + 1);
      tmp_post[cut_point + 2] = '\0';
      tmp = g_strconcat (tmp_pre, "[...]", tmp_post, NULL);

      g_free (tmp_pre);
      g_free (tmp_post);
    }
  else
    tmp = g_strdup (label);

  if (!g_utf8_validate (tmp, strlen (tmp), NULL))
    str = g_convert (tmp, strlen (tmp), "UTF-8",
		     uc_project_get_local_charset (), NULL, NULL, NULL);
  else
    str = g_strdup (tmp);

  g_free (tmp);

  return str;
}


/*
 * display messages in the progess dialog during
 * the search
 */
void
uc_application_display_search_message (const gint label_pos,
				       const gchar * message)
{
  GtkWidget *label = NULL;
  gchar *tmp = NULL;

  if (uc_search_get_exit () || !message || !search_dialog ||
      !GTK_IS_WIDGET (search_dialog))
    return;

  if (label_pos != CHECK_MESSAGE_LABEL_FIRST)
    tmp = uc_application_format_progress_dialog_label (message);
  else
    tmp = g_strdup (message);

  switch (label_pos)
    {
    case CHECK_MESSAGE_LABEL_FIRST:
      UC_GLADE_GET_WIDGET ("search_label_first", glade_search_dialog, label);
      break;

    case CHECK_MESSAGE_LABEL_SECOND:
      UC_GLADE_GET_WIDGET ("search_label_second", glade_search_dialog, label);
      break;

    default:
      g_free (tmp);
      return;
    }

  if (label_pos == CHECK_MESSAGE_LABEL_FIRST)
    gtk_label_set_markup (GTK_LABEL (label), tmp);
  else
    gtk_label_set (GTK_LABEL (label), tmp);

  g_free (tmp);
}


/*
 * display messages in the progess dialog during
 * the check process
 */
void
uc_application_display_state_message (const gint label_pos,
				      const gchar * message)
{
  GtkWidget *label = NULL;
  gchar *tmp = NULL;

  if ((!message) || (!progress_dialog) || (!GTK_IS_WIDGET (progress_dialog)))
    return;

  if (label_pos != CHECK_MESSAGE_LABEL_THIRD)
    tmp = uc_application_format_progress_dialog_label (message);
  else
    tmp = g_strdup (message);

  switch (label_pos)
    {
    case CHECK_MESSAGE_LABEL_FIRST:
      UC_GLADE_GET_WIDGET ("firstLabel", glade_progress_dialog, label);
      break;

    case CHECK_MESSAGE_LABEL_SECOND:
      UC_GLADE_GET_WIDGET ("secondLabel", glade_progress_dialog, label);
      break;

    case CHECK_MESSAGE_LABEL_THIRD:
      UC_GLADE_GET_WIDGET ("thirdLabel", glade_progress_dialog, label);
      break;

    default:
      g_free (tmp);
      return;
    }

  if (label_pos == CHECK_MESSAGE_LABEL_THIRD)
    gtk_label_set_markup (GTK_LABEL (label), tmp);
  else
    gtk_label_set (GTK_LABEL (label), tmp);

  g_free (tmp);
}


/*
 * search the header of a given url
 * in the urls list to display it
 */
gboolean
uc_application_display_informations (GtkTreeView * treeview)
{
  gint id = 0;
  UCLinkProperties *prop = NULL;
  UCStatusCode *sc = NULL;
  gchar *status_code = NULL;
  gchar *tmp_str = NULL;
  GtkTextBuffer *buffer = NULL;
  GtkWidget *text_informations = NULL;
  GtkWidget *scrolled_window_informations = NULL;

  if ((id = uc_check_treeview_get_selected_row_id ()) < 0)
    {
      uc_application_dialog_show (_
				  ("Please, select a item first."),
				  GTK_MESSAGE_WARNING);

      return TRUE;
    }

  prop = uc_lists_checked_links_lookup_by_uid (id);

  UC_GLADE_GET_WIDGET ("scrolled_window_informations", glade_main_window,
		       scrolled_window_informations);
  UC_GLADE_GET_WIDGET ("text_informations", glade_main_window,
		       text_informations);

  /* empty the buffer if the selected item is a bookmark's
   * folder */
  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_informations));
  if (prop->link_type != LINK_TYPE_BOOKMARK_FOLDER)
    {
      status_code = g_hash_table_lookup (prop->header, "Status");
      if (!(sc = uc_application_get_status_code_properties (status_code)))
	sc = uc_application_get_status_code_properties (STATUS_CODE_UNKNOWN);

      tmp_str = g_strconcat ("<b><i>", sc->message, "</i></b>", NULL);
      gtk_label_set_markup (GTK_LABEL
			    (glade_xml_get_widget
			     (glade_main_window,
			      "label_informations_message")), tmp_str);
      g_free (tmp_str);

      gtk_text_buffer_set_text (buffer, sc->description, -1);
    }
  else
    {
      gtk_label_set (GTK_LABEL
		     (glade_xml_get_widget
		      (glade_main_window, "label_informations_message")),
		     " ");
      gtk_text_buffer_set_text (buffer, " ", -1);
    }

  return TRUE;
}


/*
 * display the treview popup menu
 */
void
uc_application_treeview_activate_popup (GdkEventButton * event)
{
  GladeXML *g = NULL;
  GtkWidget *popup = NULL;
  gchar *status_code = NULL;
  gint id = 0;
  UCLinkProperties *prop = NULL;

  if ((id = uc_check_treeview_get_selected_row_id ()) < 0)
    return;

  prop = uc_lists_checked_links_lookup_by_uid (id);

  /* do not display informatino for bookmark's
   * folders */
  if (prop->link_type == LINK_TYPE_BOOKMARK_FOLDER)
    return;

  status_code = g_hash_table_lookup (prop->header, "Status");

  UC_GLADE_INIT ("treeview_popup_menu", g, popup);

  gtk_menu_popup (GTK_MENU (popup), NULL, NULL, NULL, NULL,
		  event->button, event->time);

  /* adctivate/eactivate appropriate items */
  UC_WIDGET_SET_SENSITIVE (g, "delete_link", FALSE);
  if (prop->link_type != LINK_TYPE_EMAIL)
    {
      UC_WIDGET_SET_SENSITIVE (g, "menu_check_email_mx", FALSE);
    }
  else
    {
      UC_WIDGET_SET_SENSITIVE (g, "view_online", FALSE);
    }
  if ((!uc_check_status_is_good (status_code))
      || (prop->link_type != LINK_TYPE_HREF
	  && prop->link_type != LINK_TYPE_FRAME))
    {
      UC_WIDGET_SET_SENSITIVE (g, "view_link_content", FALSE);
    }
  if (!uc_project_get_download_images_content ()
      || (prop->link_type != LINK_TYPE_IMAGE
	  || !uc_check_status_is_good (status_code)))
    {
      UC_WIDGET_SET_SENSITIVE (g, "image_preview", FALSE);
    }
  if (uc_project_get_bookmarks_file ())
    {
      UC_WIDGET_SET_SENSITIVE (g, "view_similar_link_locations", FALSE);
      UC_WIDGET_SET_SENSITIVE (g, "view_parent_page_online", FALSE);
      UC_WIDGET_SET_SENSITIVE (g, "delete_link", TRUE);
    }

  gtk_widget_show (popup);
}


/*
 * saving states of a notebook check
 * button
 */
static void
uc_application_buttons_save_setting (GtkWidget * widget, gchar * path)
{
  GList *items = NULL;
  gchar **radiosArray = NULL;
  guint size = 0;
  GString *radiosStr = g_string_new ("");

  items = uc_application_buttons_get_list (widget);
  while (items)
    {
      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (items->data)))
	{
	  g_string_append_printf (radiosStr, ",%s",
				  gtk_button_get_label
				  (GTK_BUTTON (items->data)));
	  size++;
	}
      items = items->next;
    }
  g_list_free (items);

  if (size)
    g_string_erase (radiosStr, 0, 1);

  radiosArray = g_strsplit (radiosStr->str, ",", CHECK_BUTTON_MAX_DISPLAY);

  UC_CONFIG_SET_VECTOR (path, (const gchar **) radiosArray, size);

  g_strfreev (radiosArray);
}


/*
 * -- FIXME --
 * return a GList of pointers to the check buttons
 * of the notebooks
 */
static GList *
uc_application_buttons_get_list (const GtkWidget * widget)
{
  GList *parents = NULL;
  GList *childs = NULL;
  GList *list = NULL;

  parents = gtk_container_get_children (GTK_CONTAINER (widget));
  while (parents)
    {
      if (GTK_IS_TABLE (parents->data))
	{
	  childs = gtk_container_get_children (GTK_CONTAINER (parents->data));
	  while (childs)
	    {
	      list = g_list_prepend (list, childs->data);
	      childs = g_list_next (childs);
	    }
	  g_list_free (childs);
	}
      parents = g_list_next (parents);
    }

  g_list_free (parents);

  return list;
}


/*
 * dislay items in a frame
 */
static void
uc_application_buttons_display (const GtkWidget * frame, gpointer array,
				const guint items_size,
				gpointer selected_array,
				const guint selected_items_size)
{
  GtkWidget *checkButton = NULL;
  GtkWidget *table = NULL;
  gchar **items = (gchar **) array;
  gchar **selected_items = (gchar **) selected_array;
  guint i = 0;
  guint col = 0;
  guint row = 0;
  gdouble maxRows = 0;
  gdouble maxCols = DIALOG_SETTINGS_DOCUMENT_TYPES_MAX_COLS;

  maxRows = ceil (items_size / maxCols);

  table = gtk_table_new (maxCols, maxRows, FALSE);
  gtk_widget_show (table);
  gtk_container_add (GTK_CONTAINER (frame), table);
  gtk_container_set_border_width (GTK_CONTAINER (table), 5);
  gtk_table_set_row_spacings (GTK_TABLE (table), 2);
  gtk_table_set_col_spacings (GTK_TABLE (table), 2);

  for (row = 0, i = 0; row < maxRows; row++, col = 0)
    while ((col < maxCols) && (i < items_size))
      {
	checkButton = gtk_check_button_new_with_mnemonic (items[i]);

	if (uc_utils_array_is_in
	    (items[i], selected_items, selected_items_size))
	  gtk_toggle_button_set_active
	    (GTK_TOGGLE_BUTTON (checkButton), TRUE);
	gtk_widget_show (checkButton);
	gtk_table_attach (GTK_TABLE (table), checkButton,
			  col, col + 1, row, row + 1,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	col++, i++;
      }
}


/*
 * remove all application paths
 */
void
uc_application_remove_paths (void)
{
  gchar *path = NULL;
  gchar *final_path = NULL;

  if (!
      (path =
       UC_CONFIG_GET_STRING (UC_CONFIG_PROPERTIES_GENERAL_WORKING_PATH)))
    path = g_strdup (UC_CONFIG_PROPERTIES_GENERAL_DEFAULT_WORKING_PATH);

  final_path = g_strconcat (path, "/", uc_project_get_cache_name (), NULL);

  uc_utils_rmdirs (final_path, FALSE);

  g_free (path);
  g_free (final_path);
}


/*
 * display list of elements to reject
 */
static void
uc_application_settings_dialog_display_filters_list (const gchar * glade,
						     gpointer elements,
						     guint items_size,
						     GtkTreeView ** tv,
						     GtkListStore ** model)
{
  enum
  {
    NAME_COLUMN,
    N_COLUMNS
  };
  gchar **items = (gchar **) elements;
  GtkTreeIter iter;
  GtkCellRenderer *renderer;
  guint i = 0;
  GtkWidget *scroll = NULL;

  UC_GLADE_GET_WIDGET (glade, glade_settings_dialog, scroll);

  if (*model)
    g_object_unref (*model);

  *model = gtk_list_store_new (N_COLUMNS, G_TYPE_STRING);

  if (!strlen (items[0]))
    items_size = 0;

  for (i = 0; i < items_size; i++)
    {
      gtk_list_store_append (*model, &iter);
      gtk_list_store_set (*model, &iter, NAME_COLUMN, (gchar *) items[i], -1);
    }


  *tv = (GtkTreeView *) gtk_tree_view_new_with_model ((GtkTreeModel *) *
						      model);

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (*tv),
					       -1,
					       _("Name"), renderer,
					       "text", NAME_COLUMN, NULL);
  uc_utils_set_userfriendly_treeview_column (GTK_TREE_VIEW (*tv),
					     NAME_COLUMN);

  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (*tv), TRUE);

  gtk_container_add (GTK_CONTAINER (scroll), (GtkWidget *) * tv);

  gtk_widget_show_all (scroll);
}


/*
 * return TRUE if the header page can be displayed
 */
static gboolean
uc_application_page_information_dialog_display_header_is_valid (const gchar
								* status_code)
{
  return (!uc_check_status_is_malformed (status_code) &&
	  !uc_check_status_is_timeout (status_code) &&
	  !uc_check_status_is_ignored (status_code) &&
	  !uc_check_status_is_email (status_code));
}


/*
 * display header information
 */
static void
uc_application_page_information_dialog_display_header (const
						       UCLinkProperties *
						       prop)
{
  enum
  {
    NAME_COLUMN,
    VALUE_COLUMN,
    N_COLUMNS
  };
  GtkTreeView *tv;
  GtkTreeIter iter;
  GtkListStore *model;
  GtkCellRenderer *renderer;
  GtkTextBuffer *text_buffer = NULL;
  GtkWidget *scroll = NULL;
  GtkWidget *widget = NULL;
  GHashNode *node = NULL;
  UCStatusCode *sc = NULL;
  gchar *parent_url = NULL;
  gchar *status_code = NULL;
  gchar no_value[] = " ";
  guint i = 0;

  UC_GLADE_GET_WIDGET ("general_scrolled_up", glade_page_information_dialog,
		       scroll);

  model = gtk_list_store_new (N_COLUMNS, G_TYPE_STRING, G_TYPE_STRING);

  status_code = g_hash_table_lookup (prop->header, "Status");
  if (!(sc = uc_application_get_status_code_properties (status_code)))
    sc = uc_application_get_status_code_properties (STATUS_CODE_UNKNOWN);

  /* link */
  UC_GLADE_GET_WIDGET ("general_link", glade_page_information_dialog, widget);
  text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (widget));
  gtk_text_buffer_set_text (text_buffer, prop->url, -1);
  gtk_text_view_set_buffer (GTK_TEXT_VIEW (widget), text_buffer);

  /* parent page */
  UC_GLADE_GET_WIDGET ("general_parent", glade_page_information_dialog,
		       widget);

  /* there is no parent url for a local file or bookmark */
  if (!uc_project_get_bookmarks_file ())
    parent_url = (prop->parent && prop->parent->url) ?
      prop->parent->url : uc_project_get_url ();
  else
    parent_url = no_value;

  text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (widget));
  gtk_text_buffer_set_text (text_buffer, parent_url, -1);
  gtk_text_view_set_buffer (GTK_TEXT_VIEW (widget), text_buffer);

  /* description */
  UC_GLADE_GET_WIDGET ("general_page_description",
		       glade_page_information_dialog, widget);
  text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (widget));
  gtk_text_buffer_set_text (text_buffer, sc->description, -1);
  gtk_text_view_set_buffer (GTK_TEXT_VIEW (widget), text_buffer);

  /* header */
  if (uc_application_page_information_dialog_display_header_is_valid
      (status_code))
    for (i = 0; i < prop->header->size; i++)
      for (node = prop->header->nodes[i]; node; node = node->next)
	{
	  gtk_list_store_append (model, &iter);
	  gtk_list_store_set (model, &iter,
			      NAME_COLUMN,
			      (gchar *) node->key,
			      VALUE_COLUMN, (gchar *) node->value, -1);
	}

  tv = (GtkTreeView *) gtk_tree_view_new_with_model ((GtkTreeModel *) model);
  g_object_unref (model);

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (tv,
					       -1,
					       _("Name"), renderer,
					       "text", NAME_COLUMN, NULL);
  uc_utils_set_userfriendly_treeview_column (tv, NAME_COLUMN);

  gtk_tree_view_insert_column_with_attributes (tv,
					       -1,
					       _("Value"), renderer,
					       "text", VALUE_COLUMN, NULL);
  uc_utils_set_userfriendly_treeview_column (tv, VALUE_COLUMN);

  gtk_tree_view_set_rules_hint (tv, TRUE);

  gtk_container_add (GTK_CONTAINER (scroll), GTK_WIDGET (tv));

  gtk_widget_show_all (scroll);
}


/*
 * display page information emails list
 */
static void
uc_application_page_information_dialog_display_emails (const
						       UCLinkProperties *
						       prop)
{
  enum
  {
    LABEL_COLUMN,
    URL_COLUMN,
    N_COLUMNS
  };
  GtkTreeView *tv;
  GtkTreeIter iter;
  GtkListStore *model;
  GtkCellRenderer *renderer;
  GList *item = NULL;
  GtkWidget *scroll = NULL;

  UC_GLADE_GET_WIDGET ("emails_scrolled", glade_page_information_dialog,
		       scroll);

  model = gtk_list_store_new (N_COLUMNS, G_TYPE_STRING, G_TYPE_STRING);

  item = g_list_first (prop->emails);
  while (item)
    {
      UCHTMLTag *link = (UCHTMLTag *) item->data;

      gtk_list_store_append (model, &iter);
      gtk_list_store_set (model, &iter,
			  LABEL_COLUMN, link->label,
			  URL_COLUMN, link->value, -1);

      item = g_list_next (item);
    }

  tv = (GtkTreeView *) gtk_tree_view_new_with_model ((GtkTreeModel *) model);
  g_object_unref (model);

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (tv,
					       -1,
					       _("Label"), renderer,
					       "text", LABEL_COLUMN, NULL);
  uc_utils_set_userfriendly_treeview_column (tv, LABEL_COLUMN);

  gtk_tree_view_insert_column_with_attributes (tv,
					       -1,
					       _("Email"), renderer,
					       "text", URL_COLUMN, NULL);
  uc_utils_set_userfriendly_treeview_column (tv, URL_COLUMN);

  gtk_tree_view_set_rules_hint (tv, TRUE);

  gtk_container_add (GTK_CONTAINER (scroll), GTK_WIDGET (tv));

  gtk_widget_show_all (scroll);
}


/*
 * display page information urls list
 */
static void
uc_application_page_information_dialog_display_links (const UCLinkProperties
						      * prop)
{
  enum
  {
    LABEL_COLUMN,
    URL_COLUMN,
    N_COLUMNS
  };
  GtkTreeView *tv;
  GtkTreeIter iter;
  GtkListStore *model;
  GtkCellRenderer *renderer;
  GList *item = NULL;
  GtkWidget *scroll = NULL;

  UC_GLADE_GET_WIDGET ("links_scrolled", glade_page_information_dialog,
		       scroll);

  model = gtk_list_store_new (N_COLUMNS, G_TYPE_STRING, G_TYPE_STRING);

  item = g_list_first (prop->all_links);
  while (item)
    {
      UCHTMLTag *link = (UCHTMLTag *) item->data;

      gtk_list_store_append (model, &iter);
      gtk_list_store_set (model, &iter,
			  LABEL_COLUMN, link->label,
			  URL_COLUMN, link->value, -1);

      item = g_list_next (item);
    }

  tv = (GtkTreeView *) gtk_tree_view_new_with_model ((GtkTreeModel *) model);
  g_object_unref (model);

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (tv,
					       -1,
					       _("Label"), renderer,
					       "text", LABEL_COLUMN, NULL);
  uc_utils_set_userfriendly_treeview_column (tv, LABEL_COLUMN);

  gtk_tree_view_insert_column_with_attributes (tv,
					       -1,
					       _("URL"), renderer,
					       "text", URL_COLUMN, NULL);
  uc_utils_set_userfriendly_treeview_column (tv, URL_COLUMN);

  gtk_tree_view_set_rules_hint (tv, TRUE);

  gtk_container_add (GTK_CONTAINER (scroll), GTK_WIDGET (tv));

  gtk_widget_show_all (scroll);
}


/*
 * show similar links dialog
 */
void
uc_application_view_similar_links_dialog_show (void)
{
  GtkTreeView *tv;
  GtkTreeIter iter;
  GtkListStore *model;
  GtkCellRenderer *renderer;
  GList *item = NULL;
  UCLinkProperties *prop = NULL;
  GtkWidget *scrolled_similar_links = NULL;
  gint id = 0;

  if ((id = uc_check_treeview_get_selected_row_id ()) < 0)
    {
      uc_application_dialog_show (_
				  ("Please, select a item first."),
				  GTK_MESSAGE_WARNING);
      return;
    }

  UC_GLADE_INIT ("similar_links_dialog", glade_similar_links_dialog,
		 similar_links_dialog);
  UC_GLADE_GET_WIDGET ("scrolled_similar_links", glade_similar_links_dialog,
		       scrolled_similar_links);

  model = gtk_list_store_new (1, G_TYPE_STRING);

  prop = uc_lists_checked_links_lookup_by_uid (id);

  item = g_list_first (prop->similar_links_parents);
  while (item)
    {
      gchar *str = (gchar *) item->data;

      gtk_list_store_append (model, &iter);
      gtk_list_store_set (model, &iter, 0, str, -1);

      item = g_list_next (item);
    }

  tv = (GtkTreeView *) gtk_tree_view_new_with_model ((GtkTreeModel *) model);
  g_object_unref (model);

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (tv,
					       -1, _("URL"),
					       renderer, "text", 0, NULL);
  uc_utils_set_userfriendly_treeview_column (tv, 0);

  gtk_tree_view_set_rules_hint (tv, TRUE);

  gtk_container_add (GTK_CONTAINER (scrolled_similar_links), GTK_WIDGET (tv));

  gtk_widget_show_all (similar_links_dialog);
}


/*
 * launch a web browser to view the selected
 * url
 */
void
uc_application_launch_web_browser (const gchar * url)
{
  gchar *message = NULL;
  GError *err = NULL;
  gchar *command_line = NULL;
  gchar *browser = NULL;
  gint ret = 0;

  browser =
    UC_CONFIG_GET_STRING (UC_CONFIG_PROPERTIES_GENERAL_WEB_BROWSER_PATH);
  command_line = g_strconcat (browser, " ", url, NULL);
  g_free (browser);

  ret = g_spawn_command_line_async (command_line, &err);
  g_free (command_line);

  if (!ret)
    {

      message = g_strdup_printf ("%s\n\n%s",
				 _
				 ("Cannot start your web browser; "
				  "the following error occured:"),
				 err->message);

      uc_application_dialog_show (message, GTK_MESSAGE_ERROR);

      g_clear_error (&err);
      g_free (message);
    }
}


/*
 * show view image dialog
 */
void
uc_application_view_image_dialog_show (void)
{
  gint id = 0;
  gchar *src = NULL;

  if ((id = uc_check_treeview_get_selected_row_id ()) < 0)
    {
      uc_application_dialog_show (_
				  ("Please, select a item first."),
				  GTK_MESSAGE_WARNING);

      return;
    }

  if ((src = uc_utils_convert_uid2file (id)))
    {
      GtkImage *image_preview = NULL;

      UC_GLADE_INIT ("view_image_dialog", glade_view_image_dialog,
		     view_image_dialog);

      image_preview =
	GTK_IMAGE (glade_xml_get_widget
		   (glade_view_image_dialog, "imagePreview"));

      gtk_image_set_from_file (image_preview, src);

      g_free (src);

      gtk_widget_show (view_image_dialog);
    }
  else
    uc_application_dialog_show (_("The corresponding "
				  "file is empty or does not exist "
				  "in the cache directory!"),
				GTK_MESSAGE_ERROR);
}


/*
 * show view source dialog
 */
void
uc_application_view_source_dialog_show (void)
{
  gint id = 0;
  gchar *src = NULL;

  if ((id = uc_check_treeview_get_selected_row_id ()) < 0)
    {
      uc_application_dialog_show (_
				  ("Please, select a item first."),
				  GTK_MESSAGE_WARNING);

      return;
    }

  if ((src = uc_cache_get_source (id)))
    {
      gchar *tmp = NULL;
      GtkWidget *text_view_source = NULL;
      GtkTextBuffer *buffer = NULL;

      UC_GLADE_INIT ("view_source_dialog", glade_view_source_dialog,
		     view_source_dialog);
      UC_GLADE_GET_WIDGET ("textViewSource", glade_view_source_dialog,
			   text_view_source);

      buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view_source));

      tmp = (g_utf8_validate (src, strlen (src), NULL)) ?
	g_strdup (src) :
	g_convert (src, strlen (src),
		   "UTF-8", uc_project_get_local_charset (), NULL, NULL,
		   NULL);

      gtk_text_buffer_set_text (buffer, tmp, -1);

      g_free (tmp);
      g_free (src);

      gtk_widget_show (view_source_dialog);
    }
  else
    uc_application_dialog_show (_("The corresponding "
				  "file is empty or does not exist "
				  "in the cache directory!"),
				GTK_MESSAGE_ERROR);
}


/*
 * refresh the application display
 */
void
uc_application_update_ui (void)
{
  while (gtk_events_pending ())
    gtk_main_iteration ();
}


/*
 *  free the code properties hash table
 */
static void
uc_application_status_code_properties_free (void)
{
  if (!uc_application_status_code_properties_hash)
    return;

  g_hash_table_foreach (uc_application_status_code_properties_hash,
			uc_application_status_code_properties_free_callback,
			NULL);
  g_hash_table_destroy (uc_application_status_code_properties_hash);
}


/*
 *  callback for the uc_application_status_code_properties_free () function
 */
static void
uc_application_status_code_properties_free_callback (gpointer key,
						     gpointer value,
						     gpointer user_data)
{
  UCStatusCode *sc = (UCStatusCode *) value;

  if (sc->icon_file)
    g_object_unref (sc->icon_file);
  g_free (sc->message);
  g_free (sc->description);
}

/*
 * -- FIXME --
 * this function load the properties to associate
 * with a server status code.
 * we can not use a external file (?) because of gettext.
 */
void
uc_application_status_code_properties_init (void)
{
  UCStatusCode *sc;
  GdkColor color;

  uc_application_status_code_properties_hash =
    g_hash_table_new (g_str_hash, g_str_equal);

  /* IGNORED ITEM */
  sc = g_malloc (sizeof (UCStatusCode));
  gdk_color_parse (IGNORED_LINK_BGCOLOR, &color);
  sc->icon_file =
    gdk_pixbuf_new_from_file (UC_PIXMAPS_DIR
			      "/link_status_ignored.png", NULL);
  sc->background = color;
  sc->message = g_strdup (_("Item ignored"));
  sc->description =
    g_strdup (_("You chose to ignore this item while gURLChecker "
		"was checking it."));
  g_hash_table_insert (uc_application_status_code_properties_hash,
		       STATUS_CODE_IGNORED_LINK, (gpointer) sc);

  /* UNKNOWN */
  sc = g_malloc (sizeof (UCStatusCode));
  gdk_color_parse (GOOD_LINK_BGCOLOR, &color);
  sc->icon_file =
    gdk_pixbuf_new_from_file (UC_PIXMAPS_DIR "/link_type_unknown.png", NULL);
  sc->background = color;
  sc->message = g_strdup (_("No valid information"));
  sc->description =
    g_strdup (_
	      ("gURLChecker can't interpret the status code sent "
	       "by the remote host.\nPlease, report the value of the "
	       "\"Status\" header field to the author of gURLChecker."));
  g_hash_table_insert (uc_application_status_code_properties_hash,
		       STATUS_CODE_UNKNOWN, (gpointer) sc);

  /* 200 */
  sc = g_malloc (sizeof (UCStatusCode));
  gdk_color_parse (GOOD_LINK_BGCOLOR, &color);
  sc->icon_file =
    gdk_pixbuf_new_from_file (UC_PIXMAPS_DIR "/link_status_good.png", NULL);
  sc->background = color;
  sc->message = g_strdup (_("Page available"));
  sc->description =
    g_strdup (_
	      ("No problem with this page.\nServer is up and document "
	       "can be loaded without any problem."));
  g_hash_table_insert (uc_application_status_code_properties_hash,
		       STATUS_CODE_PAGE_OK, (gpointer) sc);

  /* 301 */
  sc = g_malloc (sizeof (UCStatusCode));
  gdk_color_parse (GOOD_LINK_BGCOLOR, &color);
  sc->icon_file =
    gdk_pixbuf_new_from_file (UC_PIXMAPS_DIR "/link_status_good.png", NULL);
  sc->background = color;
  sc->message = g_strdup (_("Page moved permanently"));
  sc->description =
    g_strdup (_
	      ("No real problem with this page.\n"
	       "The requested resource just has been assigned a new permanent "
	       "URI."));
  g_hash_table_insert (uc_application_status_code_properties_hash,
		       STATUS_CODE_MOVED_PERMANENTLY, (gpointer) sc);

  /* 302 */
  sc = g_malloc (sizeof (UCStatusCode));
  gdk_color_parse (GOOD_LINK_BGCOLOR, &color);
  sc->icon_file =
    gdk_pixbuf_new_from_file (UC_PIXMAPS_DIR "/link_status_good.png", NULL);
  sc->background = color;
  sc->message = g_strdup (_("Page available (with redirection)"));
  sc->description =
    g_strdup (_
	      ("No problem with this page.\nServer is up and document "
	       "can be loaded without any problem."));
  g_hash_table_insert (uc_application_status_code_properties_hash,
		       STATUS_CODE_REDIRECTED, (gpointer) sc);

  /* 401 */
  sc = g_malloc (sizeof (UCStatusCode));
  gdk_color_parse (RESTRICTED_LINK_BGCOLOR, &color);
  sc->icon_file =
    gdk_pixbuf_new_from_file (UC_PIXMAPS_DIR
			      "/link_status_restricted.png", NULL);
  sc->background = color;
  sc->message = g_strdup (_("Restricted access"));
  sc->description =
    g_strdup (_("This page need authentication to be viewed."));
  g_hash_table_insert (uc_application_status_code_properties_hash,
		       STATUS_CODE_RESTRICTED, (gpointer) sc);

/* 403 */
  sc = g_malloc (sizeof (UCStatusCode));
  gdk_color_parse (BAD_LINK_BGCOLOR, &color);
  sc->icon_file =
    gdk_pixbuf_new_from_file (UC_PIXMAPS_DIR "/link_status_bad.png", NULL);
  sc->background = color;
  sc->message = g_strdup (_("Access forbidden"));
  sc->description =
    g_strdup (_
	      ("The remote host has refused to serve us the page.\n"
	       "Some web servers do not allow HEAD method on particular "
	       "files... and gURLChecker need to send HEAD method :-("
	       "\n\nTry to check it later, or filter this domain in the "
	       "settings"));
  g_hash_table_insert (uc_application_status_code_properties_hash,
		       "403", (gpointer) sc);

  /* 204 */
  sc = g_malloc (sizeof (UCStatusCode));
  gdk_color_parse (NO_DATA_LINK_BGCOLOR, &color);
  sc->icon_file =
    gdk_pixbuf_new_from_file (UC_PIXMAPS_DIR "/link_status_nodata.png", NULL);
  sc->background = color;
  sc->message = g_strdup (_("Page available, but with no content"));
  sc->description =
    g_strdup (_
	      ("No problem with this page, but it have no content "
	       "at all...\n\nTry to check it later."));
  g_hash_table_insert (uc_application_status_code_properties_hash,
		       STATUS_CODE_NO_DATA, (gpointer) sc);

  /* 400 */
  sc = g_malloc (sizeof (UCStatusCode));
  gdk_color_parse (BAD_LINK_BGCOLOR, &color);
  sc->icon_file =
    gdk_pixbuf_new_from_file (UC_PIXMAPS_DIR "/link_status_bad.png", NULL);
  sc->background = color;
  sc->message = g_strdup (_("Malformed URL"));
  sc->description =
    g_strdup (_
	      ("The request could not be understood by the server "
	       "due to malformed syntax."));
  g_hash_table_insert (uc_application_status_code_properties_hash,
		       "400", (gpointer) sc);

  /* 404 */
  sc = g_malloc (sizeof (UCStatusCode));
  gdk_color_parse (BAD_LINK_BGCOLOR, &color);
  sc->icon_file =
    gdk_pixbuf_new_from_file (UC_PIXMAPS_DIR "/link_status_bad.png", NULL);
  sc->background = color;
  sc->message = g_strdup (_("Page not found"));
  sc->description =
    g_strdup (_("The page does not longer exist on the server."));
  g_hash_table_insert (uc_application_status_code_properties_hash,
		       "404", (gpointer) sc);

  /* 500 */
  sc = g_malloc (sizeof (UCStatusCode));
  gdk_color_parse (BAD_LINK_BGCOLOR, &color);
  sc->icon_file =
    gdk_pixbuf_new_from_file (UC_PIXMAPS_DIR "/link_status_bad.png", NULL);
  sc->background = color;
  sc->message = g_strdup (_("Internal server error"));
  sc->description =
    g_strdup (_("The page contain bad code, or the server is "
		"misconfigured.\n\nTry to check it later."));
  g_hash_table_insert (uc_application_status_code_properties_hash,
		       "500", (gpointer) sc);

  /* 503 */
  sc = g_malloc (sizeof (UCStatusCode));
  gdk_color_parse (TIMEOUT_LINK_BGCOLOR, &color);
  sc->icon_file =
    gdk_pixbuf_new_from_file (UC_PIXMAPS_DIR
			      "/link_status_timeout.png", NULL);
  sc->background = color;
  sc->message = g_strdup (_("Service unavailable"));
  sc->description =
    g_strdup (_
	      ("The server is currently unable to handle the request "
	       "due to a temporary overloading or maintenance "
	       "of the server. It is also possible that you are using "
	       "a proxy that can not join the given host.\n\nTry "
	       "to check it later, or change your proxy settings."));
  g_hash_table_insert (uc_application_status_code_properties_hash,
		       "503", (gpointer) sc);

  /* 504 */
  sc = g_malloc (sizeof (UCStatusCode));
  gdk_color_parse (TIMEOUT_LINK_BGCOLOR, &color);
  sc->icon_file =
    gdk_pixbuf_new_from_file (UC_PIXMAPS_DIR
			      "/link_status_timeout.png", NULL);
  sc->background = color;
  sc->message = g_strdup (_("Request not completed"));
  sc->description =
    g_strdup (_
	      (" The server, while acting as a gateway or proxy, did not "
	       "receive a timely response from the upstream server specified "
	       "by the URI (e.g. HTTP, FTP, LDAP) or some other auxiliary "
	       "server (e.g. DNS) it needed to access in attempting to "
	       "complete the request."));
  g_hash_table_insert (uc_application_status_code_properties_hash,
		       "504", (gpointer) sc);


  /* 405 */
  sc = g_malloc (sizeof (UCStatusCode));
  gdk_color_parse (BAD_LINK_BGCOLOR, &color);
  sc->icon_file =
    gdk_pixbuf_new_from_file (UC_PIXMAPS_DIR "/link_status_bad.png", NULL);
  sc->background = color;
  sc->message = g_strdup (_("Method not allowed"));
  sc->description =
    g_strdup (_
	      ("The remote host has refused to serve us the page."
	       "\n\nTry to check it later."));
  g_hash_table_insert (uc_application_status_code_properties_hash,
		       "405", (gpointer) sc);

  /* TIMEOUT */
  sc = g_malloc (sizeof (UCStatusCode));
  gdk_color_parse (TIMEOUT_LINK_BGCOLOR, &color);
  sc->icon_file =
    gdk_pixbuf_new_from_file (UC_PIXMAPS_DIR
			      "/link_status_timeout.png", NULL);
  sc->background = color;
  sc->message = g_strdup (_("Receiving timout requesting the page"));
  sc->description =
    g_strdup (_
	      ("Perhaps you would change the timeout in gURLChecker "
	       "preferences. Perhaps the remote host is "
	       "too busy for the moment to responde to "
	       "gURLChecker request, or perhaps the remote host does not "
	       "exist at all.\n\nTry to check it later."));
  g_hash_table_insert (uc_application_status_code_properties_hash,
		       STATUS_CODE_TIMEOUT, (gpointer) sc);

  /* HOST UNREACHABLE */
  sc = g_malloc (sizeof (UCStatusCode));
  gdk_color_parse (TIMEOUT_LINK_BGCOLOR, &color);
  sc->icon_file =
    gdk_pixbuf_new_from_file (UC_PIXMAPS_DIR
			      "/link_status_timeout.png", NULL);
  sc->background = color;
  sc->message = g_strdup (_("Host unreachable"));
  sc->description =
    g_strdup (_
	      ("The remote host does not respond at all...\nPerhaps "
	       "you would change the timeout in gURLChecker "
	       "preferences, or perhaps it is really down."
	       "\n\nTry to check it later."));
  g_hash_table_insert (uc_application_status_code_properties_hash,
		       STATUS_CODE_HOST_IS_UNREACHABLE, (gpointer) sc);

  /* MALFORMED LINK */
  sc = g_malloc (sizeof (UCStatusCode));
  gdk_color_parse (MALFORMED_LINK_BGCOLOR, &color);
  sc->icon_file =
    gdk_pixbuf_new_from_file (UC_PIXMAPS_DIR
			      "/link_status_malformed.png", NULL);
  sc->background = color;
  sc->message = g_strdup (_("Malformed link"));
  sc->description =
    g_strdup (_("The HTML link is malformed.\n\n"
		"Perhaps SRC attribute is empty or missing or relative "
		"link is not correct?"));
  g_hash_table_insert (uc_application_status_code_properties_hash,
		       STATUS_CODE_MALFORMED_LINK, (gpointer) sc);

  /* FILE PROTOCOL */
  sc = g_malloc (sizeof (UCStatusCode));
  gdk_color_parse (FILE_ERROR_LINK_BGCOLOR, &color);
  sc->icon_file =
    gdk_pixbuf_new_from_file (UC_PIXMAPS_DIR
			      "/link_status_file_error.png", NULL);
  sc->background = color;
  sc->message = g_strdup (_("FILE:// protocol"));
  sc->description =
    g_strdup (_("File protocol detected.\n"
		"You have asked me to warn you when I find the \"FILE://\" "
		"protocol in a link."));
  g_hash_table_insert (uc_application_status_code_properties_hash,
		       STATUS_CODE_FILE_PROTO_ERROR, (gpointer) sc);

  /* BAD EMAIL LINK SYNTAX */
  sc = g_malloc (sizeof (UCStatusCode));
  gdk_color_parse (BAD_EMAIL_LINK_BGCOLOR, &color);
  sc->icon_file =
    gdk_pixbuf_new_from_file (UC_PIXMAPS_DIR
			      "/link_status_mail_bad.png", NULL);
  sc->background = color;
  sc->message = g_strdup (_("Bad email link"));
  sc->description =
    g_strdup (_
	      ("The email address specified in the link is not well "
	       "formatted."));
  g_hash_table_insert (uc_application_status_code_properties_hash,
		       STATUS_CODE_BAD_EMAIL_LINK, (gpointer) sc);

  /* BAD EMAIL LINK MX */
  sc = g_malloc (sizeof (UCStatusCode));
  gdk_color_parse (BAD_EMAIL_LINK_BGCOLOR, &color);
  sc->icon_file =
    gdk_pixbuf_new_from_file (UC_PIXMAPS_DIR
			      "/link_status_mail_bad.png", NULL);
  sc->background = color;
  sc->message = g_strdup (_("Problem with MX"));
  sc->description =
    g_strdup (_
	      ("The MX of the domain specified for this email address is "
	       "not valid or the domain itself is not valid."));
  g_hash_table_insert (uc_application_status_code_properties_hash,
		       STATUS_CODE_BAD_EMAIL_LINK_MX, (gpointer) sc);


  /* GOOD EMAIL LINK */
  sc = g_malloc (sizeof (UCStatusCode));
  gdk_color_parse (GOOD_EMAIL_LINK_BGCOLOR, &color);
  sc->icon_file =
    gdk_pixbuf_new_from_file (UC_PIXMAPS_DIR
			      "/link_status_mail_good.png", NULL);
  sc->background = color;
  sc->message = g_strdup (_("Email link ok"));
  sc->description = g_strdup (_("No problem with this email address."));
  g_hash_table_insert (uc_application_status_code_properties_hash,
		       STATUS_CODE_GOOD_EMAIL_LINK, (gpointer) sc);

}


/*
 * launch a new instance of gURLChecker
 */
void
uc_application_new_instance_launch (void)
{
  gchar *message = NULL;
  GError *err = NULL;
  gchar *argv[] = { PACKAGE_BIN_DIR "/" PACKAGE, NULL };
  gchar **envp = environ;
  GSpawnFlags flags = G_SPAWN_DO_NOT_REAP_CHILD;
  gint child_pid = 0;

  if (g_spawn_async (NULL, argv, envp, flags, NULL, NULL, &child_pid, &err))
    return;

  message = g_strdup_printf ("%s\n\n%s",
			     _
			     ("Cannot start new instance of gURLChecker; "
			      "the following error occured:"), err->message);
  uc_application_dialog_show (message, GTK_MESSAGE_ERROR);
  g_clear_error (&err);

  g_free (message);
}


/*
 * launch the gnome network preference dialog
 */
void
uc_application_proxy_launch_gnome_pref (void)
{
  gchar *message = NULL;
  GError *err = NULL;

  if (g_spawn_command_line_async ("gnome-network-preferences", &err))
    return;

  message = g_strdup_printf ("%s\n\n%s",
			     _("Cannot start network proxy "
			       "configuration program; the "
			       "following error occured:"), err->message);
  uc_application_dialog_show (message, GTK_MESSAGE_ERROR);
  g_clear_error (&err);

  g_free (message);
}


/*
 *  show the settings dialog box
 */
void
uc_application_settings_dialog_show (void)
{
  GtkWidget *widget = NULL;
  GtkWidget *widget_bis = NULL;
  gchar **items = NULL;
  gchar **selected_items = NULL;
  gchar *str = NULL;
  GtkWidget *frame = NULL;
  guint items_size = 0;
  guint value = 0;
  guint selected_items_size = 0;

  UC_GLADE_INIT ("settings_dialog", glade_settings_dialog, settings_dialog);

  /* GENERAL */

  /* network timeout */
  UC_GLADE_GET_WIDGET ("spinTimeout", glade_settings_dialog, widget);
  value = UC_CONFIG_GET_INT (UC_CONFIG_PROPERTIES_GENERAL_NETWORKING_TIMEOUT);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget), (gfloat) value);

  /* working path */
  UC_GLADE_GET_WIDGET ("workingPath", glade_settings_dialog, widget);
  UC_GLADE_GET_WIDGET ("workingPathEntry", glade_settings_dialog, widget_bis);
  str = UC_CONFIG_GET_STRING (UC_CONFIG_PROPERTIES_GENERAL_WORKING_PATH);
  if (str)
    {
      gtk_entry_set_text (GTK_ENTRY (widget_bis), str);
      gnome_file_entry_set_default_path (GNOME_FILE_ENTRY (widget), str);
    }
  else
    {
      gtk_entry_set_text (GTK_ENTRY (widget_bis),
			  UC_CONFIG_PROPERTIES_GENERAL_DEFAULT_WORKING_PATH);

      gnome_file_entry_set_default_path (GNOME_FILE_ENTRY (widget),
					 UC_CONFIG_PROPERTIES_GENERAL_DEFAULT_WORKING_PATH);
    }
  g_free (str);

  /* Web browser */
  UC_GLADE_GET_WIDGET ("web_browser", glade_settings_dialog, widget);
  UC_GLADE_GET_WIDGET ("web_browser_entry", glade_settings_dialog,
		       widget_bis);
  str = UC_CONFIG_GET_STRING (UC_CONFIG_PROPERTIES_GENERAL_WEB_BROWSER_PATH);
  if (str)
    {
      gtk_entry_set_text (GTK_ENTRY (widget_bis), str);
      gnome_file_entry_set_default_path (GNOME_FILE_ENTRY (widget), str);
    }
  else
    {
      gtk_entry_set_text (GTK_ENTRY (widget_bis),
			  UC_CONFIG_PROPERTIES_GENERAL_DEFAULT_WEB_BROWSER_PATH);

      gnome_file_entry_set_default_path (GNOME_FILE_ENTRY (widget),
					 UC_CONFIG_PROPERTIES_GENERAL_DEFAULT_WEB_BROWSER_PATH);
    }
  g_free (str);

  /* debug mode? */
  UC_GLADE_GET_WIDGET ("debug_mode", glade_settings_dialog, widget);
  value = UC_CONFIG_GET_INT (UC_CONFIG_PROPERTIES_GENERAL_DEBUG);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), (gboolean) value);

  /* dump properties? */
  UC_GLADE_GET_WIDGET ("dump_properties", glade_settings_dialog, widget);
  value =
    UC_CONFIG_GET_INT (UC_CONFIG_PROPERTIES_GENERAL_DEBUG_DUMP_PROPERTIES);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), (gboolean) value);

  /* URL */

  /* networking depth level */
  UC_GLADE_GET_WIDGET ("spinURLDepthLevel", glade_settings_dialog, widget);
  value = UC_CONFIG_GET_INT (UC_CONFIG_PROPERTIES_URL_NETWORKING_DEPTH_LEVEL);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget), (gfloat) value);

  /* retrieve images content */
  UC_GLADE_GET_WIDGET ("images_content", glade_settings_dialog, widget);
  value =
    UC_CONFIG_GET_INT (UC_CONFIG_PROPERTIES_FILTERS_RETRIEVE_IMAGES_CONTENT);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), (gboolean) value);

  /* retrieve archives content */
  UC_GLADE_GET_WIDGET ("archives_content", glade_settings_dialog, widget);
  value =
    UC_CONFIG_GET_INT
    (UC_CONFIG_PROPERTIES_FILTERS_RETRIEVE_ARCHIVES_CONTENT);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), (gboolean) value);

  /* check HTML comments */
  UC_GLADE_GET_WIDGET ("check_comments", glade_settings_dialog, widget);
  value =
    UC_CONFIG_GET_INT (UC_CONFIG_PROPERTIES_FILTERS_CHECK_HTML_COMMENTS);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), (gboolean) value);

  /* networking check wait */
  UC_GLADE_GET_WIDGET ("spinURLWait", glade_settings_dialog, widget);
  value = UC_CONFIG_GET_INT (UC_CONFIG_PROPERTIES_URL_NETWORKING_CHECK_WAIT);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget), (gfloat) value);

  /* documents to reject */
  UC_GLADE_GET_WIDGET ("frame_filters_documents", glade_settings_dialog,
		       frame);
  items = UC_CONFIG_PROPERTIES_GET_DEFAULT_CONTENT_TYPES_DOCUMENTS;
  items_size = UC_CONFIG_PROPERTIES_DEFAULT_CONTENT_TYPES_DOCUMENTS_LEN;
  UC_CONFIG_GET_VECTOR
    (UC_CONFIG_PROPERTIES_URL_CONTENT_TYPES_DOCUMENTS,
     &selected_items, &selected_items_size);
  uc_application_buttons_display ((GtkWidget *) frame, items,
				  items_size, selected_items,
				  selected_items_size);
  g_strfreev (items);

  /* images to reject */
  UC_GLADE_GET_WIDGET ("frame_filters_images", glade_settings_dialog, frame);
  items = UC_CONFIG_PROPERTIES_GET_DEFAULT_CONTENT_TYPES_IMAGES;
  items_size = UC_CONFIG_PROPERTIES_DEFAULT_CONTENT_TYPES_IMAGES_LEN;
  UC_CONFIG_GET_VECTOR (UC_CONFIG_PROPERTIES_URL_CONTENT_TYPES_IMAGES,
			&selected_items, &selected_items_size);
  uc_application_buttons_display (frame, items, items_size,
				  selected_items, selected_items_size);
  g_strfreev (items);

  /* directories to reject */
  UC_CONFIG_GET_VECTOR
    (UC_CONFIG_PROPERTIES_FILTERS_REJECT_DIRECTORIES, &items, &items_size);
  uc_application_settings_dialog_display_filters_list
    ("scrolled_filters_directories", items, items_size,
     &treeview_filter_directories, &treestore_filter_directories);
  g_strfreev (items);

  /* domains to reject */
  UC_CONFIG_GET_VECTOR (UC_CONFIG_PROPERTIES_FILTERS_REJECT_DOMAINS,
			&items, &items_size);
  uc_application_settings_dialog_display_filters_list
    ("scrolled_filters_domains", items, items_size,
     &treeview_filter_domains, &treestore_filter_domains);
  g_strfreev (items);

  /* accept/not accept file protocol & consider/not consider FILE 
   * protocol like a error */
  UC_GLADE_GET_WIDGET ("radio_file_protocol", glade_settings_dialog, widget);
  if (UC_CONFIG_GET_INT (UC_CONFIG_PROPERTIES_FILTERS_PROTOCOLS_FILE))
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
  else
    {
      UC_GLADE_GET_WIDGET ("radio_file_is_error", glade_settings_dialog,
			   widget);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
    }

  /* email check */
  UC_GLADE_GET_WIDGET ("check_email_address", glade_settings_dialog, widget);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
				(gboolean)
				UC_CONFIG_GET_INT
				(UC_CONFIG_PROPERTIES_FILTERS_PROTOCOLS_MAILTO));

  /* email MX check */
  UC_GLADE_GET_WIDGET ("check_email_mx", glade_settings_dialog, widget);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
				(gboolean)
				UC_CONFIG_GET_INT
				(UC_CONFIG_PROPERTIES_FILTERS_PROTOCOLS_MAILTO_CHECK_MX));

  gtk_widget_set_sensitive (GTK_WIDGET (widget),
			    (gboolean)
			    UC_CONFIG_GET_INT
			    (UC_CONFIG_PROPERTIES_FILTERS_PROTOCOLS_MAILTO));

  gtk_widget_show (settings_dialog);
}


/*
 *  retrieving user input data settings
 */
gboolean
uc_application_settings_get_data (void)
{
  GtkWidget *widget = NULL;
  gchar *str = NULL;

  /* GENERAL */

  /* network timeout */
  UC_GLADE_GET_WIDGET ("spinTimeout", glade_settings_dialog, widget);
  UC_CONFIG_SET_INT (UC_CONFIG_PROPERTIES_GENERAL_NETWORKING_TIMEOUT,
		     (gint)
		     gtk_spin_button_get_value (GTK_SPIN_BUTTON (widget)));

  /* working path */
  UC_GLADE_GET_WIDGET ("workingPath", glade_settings_dialog, widget);
  if (!
      (str =
       gnome_file_entry_get_full_path (GNOME_FILE_ENTRY (widget), FALSE)))
    {
      uc_application_dialog_show (_
				  ("Please choose a working path for "
				   "gURLChecker."), GTK_MESSAGE_ERROR);
      return FALSE;
    }

  if (!uc_utils_working_path_is_valid (str))
    {
      uc_application_dialog_show (_
				  ("The path you entered is not a "
				   "directory or your don't have appropriate "
				   "rights on it. Please choose a valid "
				   "working path for gURLChecker."),
				  GTK_MESSAGE_ERROR);
      return FALSE;
    }

  UC_CONFIG_SET_STRING (UC_CONFIG_PROPERTIES_GENERAL_WORKING_PATH, str);

  /* web browser */
  UC_GLADE_GET_WIDGET ("web_browser", glade_settings_dialog, widget);
  if (!
      (str =
       gnome_file_entry_get_full_path (GNOME_FILE_ENTRY (widget), FALSE)))
    {
      uc_application_dialog_show (_
				  ("Please choose a web browser for viewing "
				   "urls."), GTK_MESSAGE_ERROR);
      return FALSE;
    }
  if (!uc_utils_file_is_executable (str))
    {
      uc_application_dialog_show (_
				  ("The file you entered is not correct "
				   "or your don't have appropriate "
				   "rights on it. Please choose a valid "
				   "application for viewing urls."),
				  GTK_MESSAGE_ERROR);
      return FALSE;
    }

  UC_CONFIG_SET_STRING (UC_CONFIG_PROPERTIES_GENERAL_WEB_BROWSER_PATH, str);

  /* active debug mode? */
  UC_GLADE_GET_WIDGET ("debug_mode", glade_settings_dialog, widget);
  UC_CONFIG_SET_INT (UC_CONFIG_PROPERTIES_GENERAL_DEBUG, (guint)
		     gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
						   (widget)));

  /* dump properties? */
  UC_GLADE_GET_WIDGET ("dump_properties", glade_settings_dialog, widget);
  UC_CONFIG_SET_INT (UC_CONFIG_PROPERTIES_GENERAL_DEBUG_DUMP_PROPERTIES,
		     (guint)
		     gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
						   (widget)));

  /* URL */

  /* networking depth level */
  UC_GLADE_GET_WIDGET ("spinURLDepthLevel", glade_settings_dialog, widget);
  UC_CONFIG_SET_INT (UC_CONFIG_PROPERTIES_URL_NETWORKING_DEPTH_LEVEL,
		     (gint)
		     gtk_spin_button_get_value (GTK_SPIN_BUTTON (widget)));

  /* retrieving images content */
  UC_GLADE_GET_WIDGET ("images_content", glade_settings_dialog, widget);
  UC_CONFIG_SET_INT (UC_CONFIG_PROPERTIES_FILTERS_RETRIEVE_IMAGES_CONTENT,
		     (guint)
		     gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
						   (widget)));

  /* retrieving archives content */
  UC_GLADE_GET_WIDGET ("archives_content", glade_settings_dialog, widget);
  UC_CONFIG_SET_INT
    (UC_CONFIG_PROPERTIES_FILTERS_RETRIEVE_ARCHIVES_CONTENT, (guint)
     gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)));

  /* check HTML comments */
  UC_GLADE_GET_WIDGET ("check_comments", glade_settings_dialog, widget);
  UC_CONFIG_SET_INT (UC_CONFIG_PROPERTIES_FILTERS_CHECK_HTML_COMMENTS,
		     (guint)
		     gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
						   (widget)));

  /* networking check wait */
  UC_GLADE_GET_WIDGET ("spinURLWait", glade_settings_dialog, widget);
  UC_CONFIG_SET_INT (UC_CONFIG_PROPERTIES_URL_NETWORKING_CHECK_WAIT,
		     (gint)
		     gtk_spin_button_get_value (GTK_SPIN_BUTTON (widget)));

  /* documents to reject */
  UC_GLADE_GET_WIDGET ("frame_filters_documents", glade_settings_dialog,
		       widget);
  uc_application_buttons_save_setting (widget,
				       UC_CONFIG_PROPERTIES_URL_CONTENT_TYPES_DOCUMENTS);

  /* images to reject */
  UC_GLADE_GET_WIDGET ("frame_filters_images", glade_settings_dialog, widget);
  uc_application_buttons_save_setting (widget,
				       UC_CONFIG_PROPERTIES_URL_CONTENT_TYPES_IMAGES);

  /* directories filter */
  uc_application_filters_save_settings (treeview_filter_directories,
					UC_CONFIG_PROPERTIES_FILTERS_REJECT_DIRECTORIES);

  /* domains filter */
  uc_application_filters_save_settings (treeview_filter_domains,
					UC_CONFIG_PROPERTIES_FILTERS_REJECT_DOMAINS);

  /* check the FILE protocol & accept/not accept FILE protocol */
  UC_GLADE_GET_WIDGET ("radio_file_protocol", glade_settings_dialog, widget);
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
    {
      UC_CONFIG_SET_INT (UC_CONFIG_PROPERTIES_FILTERS_PROTOCOLS_FILE, 1);
      UC_CONFIG_SET_INT
	(UC_CONFIG_PROPERTIES_FILTERS_PROTOCOLS_FILE_ERROR, 0);
    }
  else
    {
      UC_CONFIG_SET_INT (UC_CONFIG_PROPERTIES_FILTERS_PROTOCOLS_FILE, 0);
      UC_CONFIG_SET_INT
	(UC_CONFIG_PROPERTIES_FILTERS_PROTOCOLS_FILE_ERROR, 1);
    }

  /* check emails */
  UC_GLADE_GET_WIDGET ("check_email_address", glade_settings_dialog, widget);
  UC_CONFIG_SET_INT (UC_CONFIG_PROPERTIES_FILTERS_PROTOCOLS_MAILTO,
		     (guint)
		     gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
						   (widget)));
  /* check MX emails */
  UC_GLADE_GET_WIDGET ("check_email_mx", glade_settings_dialog, widget);
  UC_CONFIG_SET_INT
    (UC_CONFIG_PROPERTIES_FILTERS_PROTOCOLS_MAILTO_CHECK_MX,
     (guint) gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)));


  gnome_config_sync ();

  return TRUE;
}


/*
 * save filters settings
 */
static void
uc_application_filters_save_settings (GtkTreeView * treeview,
				      const gchar * path)
{
  GtkTreeModel *tree_store = NULL;
  gchar **filters_vector = NULL;
  GtkTreeIter iter;
  GString *filters = g_string_new ("");
  guint size = 0;

  tree_store = gtk_tree_view_get_model (treeview);
  if (gtk_tree_model_get_iter_first (tree_store, &iter))
    do
      {
	GValue *value = NULL;

	value = g_new0 (GValue, 1);
	gtk_tree_model_get_value (tree_store, &iter, 0, value);

	g_string_append_printf (filters, ",%s",
				(gchar *) g_value_get_string (value));

	g_value_unset (value);
	size++;
      }
    while (gtk_tree_model_iter_next (tree_store, &iter));

  if (size)
    g_string_erase (filters, 0, 1);

  filters_vector = g_strsplit (filters->str, ",", -1);
  UC_CONFIG_SET_VECTOR (path, (const gchar **) filters_vector, size);
  g_strfreev (filters_vector);
}


/*
 * show the progess dialog
 */
void
uc_application_progress_dialog_show (void)
{
  GtkWidget *widget = NULL;
  guint value = 0;

  UC_GLADE_INIT ("progress_dialog", glade_progress_dialog, progress_dialog);

  UC_GLADE_GET_WIDGET ("dynamic_values_time_between", glade_progress_dialog,
		       widget);
  value = UC_CONFIG_GET_INT (UC_CONFIG_PROPERTIES_URL_NETWORKING_CHECK_WAIT);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget), (gfloat) value);

  UC_GLADE_GET_WIDGET ("dynamic_values_timeout", glade_progress_dialog,
		       widget);
  value = UC_CONFIG_GET_INT (UC_CONFIG_PROPERTIES_GENERAL_NETWORKING_TIMEOUT);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget), (gfloat) value);

  gtk_widget_show (progress_dialog);
}


/*
 *  show the about box
 */
void
uc_application_about_dialog_show (void)
{
  GtkWidget *about_dialog = NULL;
  GdkPixbuf *pixbuf = NULL;
  const gchar *authors[] = {
    "Emmanuel Saracco <esaracco@noos.fr>",
    "",
    _("Graphic arts:"),
    "Jan Henrik Helmers <jh@helmers.org>",
    NULL
  };
  const gchar *documentors[] = {
    _("Daniel Pecos <dani@netpecos.org>, manpage"),
    NULL
  };
  gchar *translators =
    _("Takeshi Aihana <aihana@gnome.gr.jp>, Japanese\n"
      "Alexander Hunziker <alex.hunziker@rega-sense.ch>, German\n"
      "Florent Jugla <juglaf@wanadoo.fr>, Esperanto\n"
      "Daniel Pecos <dani@netpecos.org>, Spanish\n"
      "Gunner Poulsen <gp@lyngbjerggaardskolen.dk>, Danish\n"
      "Emmanuel Saracco <esaracco@noos.fr>, French\n");

  pixbuf = gdk_pixbuf_new_from_file (UC_PIXMAPS_DIR "/gURLChecker.png", NULL);

  about_dialog =
    gnome_about_new ("gURLChecker",
		     VERSION,
		     "(C) 2002,2003 Emmanuel Saracco <esaracco@noos.fr>",
		     _("gURLChecker is a web links checker.\n"
		       "It can work on a whole site, a single local page "
		       "or a browser bookmarks file"),
		     authors, documentors, translators, pixbuf);

  if (pixbuf)
    g_object_unref (pixbuf);

  gtk_window_set_destroy_with_parent (GTK_WINDOW (about_dialog), TRUE);
  gtk_widget_show (about_dialog);
}


/*
 *  print a yes/no choice dialog
 */
gboolean
uc_application_dialog_yes_no_show (const gchar * message,
				   const GtkMessageType msg_type)
{
  gint ret = 0;

  GtkWidget *dialog = gtk_message_dialog_new (NULL,
					      GTK_DIALOG_MODAL |
					      GTK_DIALOG_DESTROY_WITH_PARENT,
					      msg_type,
					      GTK_BUTTONS_YES_NO,
					      message);

  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
  gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
  ret = gtk_dialog_run (GTK_DIALOG (dialog));
  gtk_widget_destroy (dialog);

  return (ret == GTK_RESPONSE_YES);
}


/*
 *  print a alert box
 */
void
uc_application_dialog_show (const gchar * message,
			    const GtkMessageType msg_type)
{
  GtkWidget *dialog = gtk_message_dialog_new (NULL,
					      GTK_DIALOG_MODAL |
					      GTK_DIALOG_DESTROY_WITH_PARENT,
					      msg_type,
					      GTK_BUTTONS_OK,
					      message);

  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
  gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
  gtk_dialog_run (GTK_DIALOG (dialog));
  gtk_widget_destroy (dialog);
}


/*
 *  show the new web project dialog box
 */
void
uc_application_new_web_dialog_show (void)
{
  GtkWidget *widget = NULL;

  UC_GLADE_INIT ("new_web_project_dialog", glade_new_project_dialog,
		 new_project_dialog);

  UC_GLADE_GET_WIDGET ("project_url", glade_new_project_dialog, widget);

  if (uc_project_get_url ())
    gtk_entry_set_text (GTK_ENTRY
			(gnome_entry_gtk_entry (GNOME_ENTRY (widget))),
			uc_project_get_url ());

  gtk_widget_show_all (new_project_dialog);
}


/*
 *  show the new bookmarks project dialog box
 */
void
uc_application_new_bookmarks_dialog_show (void)
{
  GtkWidget *widget = NULL;

  UC_GLADE_INIT ("new_bookmarks_project_dialog", glade_new_project_dialog,
		 new_project_dialog);

  UC_GLADE_GET_WIDGET ("project_bookmarks_file", glade_new_project_dialog,
		       widget);

  if (uc_project_get_bookmarks_file ())
    gnome_file_entry_set_default_path (GNOME_FILE_ENTRY (widget),
				       uc_project_get_bookmarks_file ());

  gtk_widget_show_all (new_project_dialog);
}


/*
 * add a domain filter
 */
void
uc_application_add_filter_domain_add (void)
{
  GtkWidget *entry = NULL;
  gchar *tmp = NULL;

  UC_GLADE_GET_WIDGET ("entry_add_filter_domain",
		       glade_add_filter_domain_dialog, entry);

  if ((tmp = (gchar *) gtk_entry_get_text (GTK_ENTRY (entry))) &&
      g_strstrip (tmp) && strlen (tmp))
    {
      GtkTreeIter iter;

      /* very basic check of the domain name syntax */
      if ((strlen (tmp) < 4) ||
	  (!strchr (tmp, '.')) ||
	  (tmp[strlen (tmp) - 1] == '.') ||
	  (tmp[strlen (tmp) - 2] == '.') || (tmp[0] == '.'))
	uc_application_dialog_show (_
				    ("The domain you entered "
				     "seems not to be valid."),
				    GTK_MESSAGE_ERROR);
      else
	{
	  gtk_list_store_append (treestore_filter_domains, &iter);
	  gtk_list_store_set (treestore_filter_domains, &iter, 0, tmp, -1);
	  gtk_widget_destroy (add_filter_domain_dialog);
	}
    }
  else
    uc_application_dialog_show (_
				("Please, enter a domain name."),
				GTK_MESSAGE_ERROR);
}


/*
 * set the iterator for the current
 * selection in a given treeview
 */
gboolean
uc_application_treeview_get_selected_iter (const GtkTreeView * tv,
					   GtkTreeIter * iter)
{
  GtkTreeModel *model = NULL;
  GtkTreeSelection *select = gtk_tree_view_get_selection (GTK_TREE_VIEW (tv));
  gint ret = -1;

  if (gtk_tree_selection_get_selected (select, &model, iter))
    {
      gtk_tree_model_get (model, iter, 0, &ret, -1);

      return TRUE;
    }

  return FALSE;;
}


/*
 *  remove a domain filter
 */
void
uc_application_add_filter_domain_remove (void)
{
  GtkTreeIter iter;

  if (uc_application_treeview_get_selected_iter
      (treeview_filter_domains, &iter))
    gtk_list_store_remove (treestore_filter_domains, &iter);
  else
    uc_application_dialog_show (_
				("Please, select a item first."),
				GTK_MESSAGE_ERROR);
}


/*
 *  add a directory filter
 */
void
uc_application_add_filter_directory_add (void)
{
  GtkWidget *entry = NULL;
  gchar *tmp = NULL;

  UC_GLADE_GET_WIDGET ("entry_add_filter_directory",
		       glade_add_filter_directory_dialog, entry);

  if ((tmp = (gchar *) gtk_entry_get_text (GTK_ENTRY (entry))) &&
      g_strstrip (tmp) && strlen (tmp))
    {
      GtkTreeIter iter;
      gchar *tmp_path = NULL;

      /* force directory to be "slashed" */
      if (tmp[strlen (tmp) - 1] == '/')
	tmp[strlen (tmp) - 1] = '\0';
      tmp_path = (tmp[0] == '/') ?
	g_strconcat (tmp, "/", NULL) : g_strconcat ("/", tmp, "/", NULL);

      gtk_list_store_append (treestore_filter_directories, &iter);
      gtk_list_store_set (treestore_filter_directories, &iter, 0,
			  tmp_path, -1);
      gtk_widget_destroy (add_filter_directory_dialog);
      g_free (tmp_path);
    }
  else
    uc_application_dialog_show (_
				("Please, enter a directory name."),
				GTK_MESSAGE_ERROR);
}


/*
 *  remove a directory filter
 */
void
uc_application_add_filter_directory_remove (void)
{
  GtkTreeIter iter;

  if (uc_application_treeview_get_selected_iter
      (treeview_filter_directories, &iter))
    gtk_list_store_remove (treestore_filter_directories, &iter);
  else
    uc_application_dialog_show (_
				("Please, select a item first."),
				GTK_MESSAGE_ERROR);
}


/*
 * retreive the string to search and the search
 * properties
 */
void
uc_application_search_get_data (void)
{
  GtkWidget *widget = NULL;
  gchar *tmp = NULL;

  UC_GLADE_GET_WIDGET ("search_string", glade_new_search_dialog, widget);
  tmp = (gchar *)
    gtk_entry_get_text (GTK_ENTRY
			(gnome_entry_gtk_entry (GNOME_ENTRY (widget))));

  if (tmp)
    {
      g_strstrip (tmp);

      if (strlen (tmp))
	{
	  gnome_entry_append_history (GNOME_ENTRY (widget), TRUE, tmp);

	  uc_search_free ();
	  searchState.string = g_strdup (tmp);

	  /* FIXME
	     searchState.case_sensitive =
	     (gtk_toggle_button_get_active
	     (GTK_TOGGLE_BUTTON
	     (glade_xml_get_widget
	     (glade_new_search_dialog, "search_case sensitive"))));
	   */
	  searchState.case_sensitive = TRUE;
	  //

	  searchState.pages_content =
	    (gtk_toggle_button_get_active
	     (GTK_TOGGLE_BUTTON
	      (glade_xml_get_widget
	       (glade_new_search_dialog, "search_pages_content"))));
	  searchState.links_name =
	    (gtk_toggle_button_get_active
	     (GTK_TOGGLE_BUTTON
	      (glade_xml_get_widget
	       (glade_new_search_dialog, "search_links_name"))));
	  searchState.meta_content =
	    (gtk_toggle_button_get_active
	     (GTK_TOGGLE_BUTTON
	      (glade_xml_get_widget
	       (glade_new_search_dialog, "search_meta_content"))));
	  searchState.emails =
	    (gtk_toggle_button_get_active
	     (GTK_TOGGLE_BUTTON
	      (glade_xml_get_widget
	       (glade_new_search_dialog, "search_emails"))));

	  gtk_widget_destroy (new_search_dialog);
	  uc_application_update_ui ();
	  uc_search_begin ();

	  return;
	}
    }

  uc_application_dialog_show (_("Please, enter a string to search for."),
			      GTK_MESSAGE_ERROR);
}


/*
 *  retrieving url to check and some check
 *  properties
 */
void
uc_application_get_web_project_data (void)
{
  GtkWidget *widget = NULL;
  gchar *tmp = NULL;

  UC_GLADE_GET_WIDGET ("project_url", glade_new_project_dialog, widget);
  tmp = (gchar *)
    gtk_entry_get_text (GTK_ENTRY
			(gnome_entry_gtk_entry (GNOME_ENTRY (widget))));

  if (tmp)
    {
      g_strstrip (tmp);

      if (strlen (tmp))
	{
	  gchar *tmp_url = uc_url_add_protocol (PROTOCOL_HTTP, tmp);

	  if (uc_url_is_valid (tmp_url))
	    {
	      /* add the given url in the combo history */
	      gnome_entry_append_history (GNOME_ENTRY (widget), TRUE, tmp);

	      /* only check this directory and its subdirectories? */
	      widget = glade_xml_get_widget
		(glade_new_project_dialog, "check_chroot");
	      uc_project_set_check_chroot ((guint)
					   gtk_toggle_button_get_active
					   (GTK_TOGGLE_BUTTON (widget)));

	      /* only check local links */
	      widget = glade_xml_get_widget
		(glade_new_project_dialog, "limit_local");
	      uc_project_set_limit_local ((guint)
					  gtk_toggle_button_get_active
					  (GTK_TOGGLE_BUTTON (widget)));

	      uc_project_set_url (tmp_url);

	      uc_project_new ();
	      gtk_widget_destroy (new_project_dialog);
	      uc_application_update_ui ();
	      uc_web_site_begin_check ();

	      return;
	    }

	  g_free (tmp_url);
	}
    }

  uc_application_dialog_show (_("Please, check that you "
				"entered the URL correctly and try again."),
			      GTK_MESSAGE_ERROR);
}


/*
 *  retrieving bookmarks file to check
 */
void
uc_application_get_bookmarks_project_data (void)
{
  GtkWidget *widget = NULL;
  gchar *file = NULL;

  UC_GLADE_GET_WIDGET ("project_bookmarks_file", glade_new_project_dialog,
		       widget);
  if (!
      (file =
       gnome_file_entry_get_full_path (GNOME_FILE_ENTRY (widget), FALSE)))
    {
      uc_application_dialog_show (_
				  ("Please choose a bookmarks file to check."),
				  GTK_MESSAGE_ERROR);
      return;
    }

  if (!g_file_test (file, G_FILE_TEST_EXISTS))
    {
      uc_application_dialog_show (_
				  ("The file you entered is not correct "
				   "or your don't have appropriate "
				   "rights on it. Please choose a valid "
				   "bookmarks file."), GTK_MESSAGE_ERROR);
      return;
    }

  /* for the moment we accept only XBEL format */
  if (!uc_bookmarks_format_is_xbel (file))
    {
      uc_application_dialog_show (_
				  ("The format of the selected file is not "
				   "correct.\n"
				   "For the moment, gURLChecker can deal "
				   "only with XBEL format."),
				  GTK_MESSAGE_ERROR);

      return;
    }

  uc_project_set_bookmarks_file (file);

  uc_project_new ();
  gtk_widget_destroy (new_project_dialog);
  uc_application_update_ui ();
  uc_bookmarks_begin_check ();
}


/*
 * draw the two principals main frames
 * for displaying urls list and headers
 */
void
uc_application_draw_main_frames ()
{
  GtkWidget *main_frames = NULL;
  GtkWidget *right_frames = NULL;
  GtkWidget *urls_list = NULL;
  GtkWidget *scrolled_window = NULL;

  UC_GLADE_GET_WIDGET ("main_frames", glade_main_window, main_frames);
  UC_GLADE_GET_WIDGET ("right_frames", glade_main_window, right_frames);
  UC_GLADE_GET_WIDGET ("scrolled_window", glade_main_window, scrolled_window);

  UC_GET_WIDGET ("urls_list", main_window, urls_list);
  if (urls_list)
    gtk_widget_destroy (urls_list);

  urls_list = gtk_tree_view_new ();
  gtk_widget_show (urls_list);
  gtk_container_add (GTK_CONTAINER (scrolled_window), urls_list);
  UC_SET_WIDGET ("urls_list", main_window, urls_list);

  g_signal_connect (G_OBJECT (urls_list), "cursor_changed",
		    G_CALLBACK (on_urlsList_cursorChanged), NULL);

  g_signal_connect (G_OBJECT (urls_list), "button_press_event",
		    G_CALLBACK (on_urlList_mouseClicked), NULL);

  g_signal_connect (G_OBJECT (urls_list), "button_release_event",
		    G_CALLBACK (on_urlList_mouseClicked), NULL);

  uc_application_resize ();

  gtk_widget_show (right_frames);
  gtk_widget_show (main_frames);
}


/*
 * reorganize all the application
 * frames
 */
void
uc_application_resize (void)
{
  GtkWidget *main_frames = NULL;
  GtkWidget *right_frames = NULL;
  guint w = 0;
  guint h = 0;

  UC_GLADE_GET_WIDGET ("main_frames", glade_main_window, main_frames);
  UC_GLADE_GET_WIDGET ("right_frames", glade_main_window, right_frames);

  gtk_window_get_size (GTK_WINDOW (main_window), &w, &h);
  gtk_paned_set_position (GTK_PANED (main_frames), (guint) h / 2);
  gtk_paned_set_position (GTK_PANED (right_frames),
			  (guint) ((w / 2) + (w / 20)));
}


/*
 * return properties for a given status code
 */
UCStatusCode *
uc_application_get_status_code_properties (const gchar * status_code)
{
  return g_hash_table_lookup
    (uc_application_status_code_properties_hash, status_code);
}


/*
 *  do a clean application exit
 */
void
uc_application_quit (void)
{
  if (uc_project_get_save_bookmarks ())
    uc_application_save_bookmarks ();

  uc_application_clean_data ();

  gtk_exit (0);
}


/*
 *  general clean up routine call at application
 *  exit
 */
static void
uc_application_clean_data (void)
{
  uc_application_remove_paths ();
  uc_project_free (UC_PROJECT_TYPE_ALL);
  uc_search_free ();
  uc_lists_checked_links_free ();
  uc_lists_already_checked_free ();
  uc_application_status_code_properties_free ();

  if (treestore_filter_directories)
    g_object_unref (treestore_filter_directories);
  if (treestore_filter_domains)
    g_object_unref (treestore_filter_domains);
  if (treestore_search)
    g_object_unref (treestore_search);
  if (treestore)
    g_object_unref (treestore);

  gnome_config_sync ();
}

/*
 *  gURLChecker initialization
 */
void
uc_application_init (gchar * url)
{
  cancel_dialog = NULL;
  uc_application_menu_init ();
  uc_application_status_code_properties_init ();
  uc_application_settings_init ();
  uc_project_new ();
  uc_check_reset ();
  uc_application_draw_main_frames ();
  uc_application_update_ui ();

  /* if user specified a url to check on
   * command line, go and check it */
  if (url)
    {
      uc_project_set_url (url);
      uc_web_site_begin_check ();
    }
}


/*
 *  create the temporaries gURLChecker directories
 */
void
uc_application_make_paths (void)
{
  gchar *path = NULL;
  gchar *final_path = NULL;

  if (!
      (path =
       UC_CONFIG_GET_STRING (UC_CONFIG_PROPERTIES_GENERAL_WORKING_PATH)))
    path = g_strdup (UC_CONFIG_PROPERTIES_GENERAL_DEFAULT_WORKING_PATH);

  final_path = g_strconcat (path, "/", uc_project_get_cache_name (), NULL);

  uc_utils_mkdirs (final_path, TRUE);

  g_free (path);
  g_free (final_path);
}


/*
 * set the defaults properties the first time
 * gURLChecker is launched
 */
static void
uc_application_settings_init (void)
{
  gchar *str = NULL;

  /* networking timeout */
  if (!UC_CONFIG_GET_INT (UC_CONFIG_PROPERTIES_GENERAL_NETWORKING_TIMEOUT))
    UC_CONFIG_SET_INT (UC_CONFIG_PROPERTIES_GENERAL_NETWORKING_TIMEOUT,
		       CHECK_TIMEOUT_DEFAULT);

  /* excluded documents extensions */
  if (!
      (str =
       UC_CONFIG_GET_STRING
       (UC_CONFIG_PROPERTIES_URL_CONTENT_TYPES_DOCUMENTS)))
    UC_CONFIG_SET_STRING
      (UC_CONFIG_PROPERTIES_URL_CONTENT_TYPES_DOCUMENTS, "");
  else
    g_free (str);

  /* excluded images extensions */
  if (!
      (str =
       UC_CONFIG_GET_STRING (UC_CONFIG_PROPERTIES_URL_CONTENT_TYPES_IMAGES)))
    UC_CONFIG_SET_STRING (UC_CONFIG_PROPERTIES_URL_CONTENT_TYPES_IMAGES, "");
  else
    g_free (str);

  /* excluded directories */
  if (!
      (str =
       UC_CONFIG_GET_STRING
       (UC_CONFIG_PROPERTIES_FILTERS_REJECT_DIRECTORIES)))
    UC_CONFIG_SET_STRING
      (UC_CONFIG_PROPERTIES_FILTERS_REJECT_DIRECTORIES, "");
  else
    g_free (str);

  /* excluded domains */
  if (!
      (str =
       UC_CONFIG_GET_STRING (UC_CONFIG_PROPERTIES_FILTERS_REJECT_DOMAINS)))
    UC_CONFIG_SET_STRING (UC_CONFIG_PROPERTIES_FILTERS_REJECT_DOMAINS, "");
  else
    g_free (str);

  /* working path */
  if (!
      (str =
       UC_CONFIG_GET_STRING (UC_CONFIG_PROPERTIES_GENERAL_WORKING_PATH)))
    UC_CONFIG_SET_STRING
      (UC_CONFIG_PROPERTIES_GENERAL_WORKING_PATH,
       UC_CONFIG_PROPERTIES_GENERAL_DEFAULT_WORKING_PATH);
  else
    g_free (str);

  /* depth level */
  if (!UC_CONFIG_GET_INT (UC_CONFIG_PROPERTIES_URL_NETWORKING_DEPTH_LEVEL))
    UC_CONFIG_SET_INT (UC_CONFIG_PROPERTIES_URL_NETWORKING_DEPTH_LEVEL,
		       DEPTH_LEVEL_DEFAULT);

  /* check wait */
  if (!UC_CONFIG_GET_INT (UC_CONFIG_PROPERTIES_URL_NETWORKING_CHECK_WAIT))
    UC_CONFIG_SET_INT
      (UC_CONFIG_PROPERTIES_URL_NETWORKING_CHECK_WAIT, CHECK_WAIT_DEFAULT);

  gnome_config_sync ();
}


/*
 * active or deactive a main menu and all his
 * items
 */
void
uc_application_menu_set_sensitive_all (const gchar * name, const gboolean val)
{
  guint i = 0;
  const gchar *search_items[] = {
    "search",
    "find",
    NULL
  };
  const gchar *display_items[] = {
    "display",
    "display_all",
    "display_goodlinks",
    "display_badlinks",
    "display_malformedlinks",
    "display_timeouts",
    "display_collapse_all",
    "display_expand_all",
    NULL
  };

  if (!strcmp ("display", name))
    while (display_items[i])
      UC_WIDGET_SET_SENSITIVE (glade_main_window, display_items[i++], val);
  else if (!strcmp ("search", name))
    while (search_items[i])
      UC_WIDGET_SET_SENSITIVE (glade_main_window, search_items[i++], val);
}


/*
 * first menu items management
 */
static void
uc_application_menu_init (void)
{
  /* no treeview, then no display options */
  uc_application_menu_set_sensitive_all ("display", FALSE);
  uc_application_menu_set_sensitive_all ("search", FALSE);

  uc_project_set_save_bookmarks (FALSE);

  /* FIXME 
   * for the moment we are not dealing with local files */
  UC_WIDGET_SET_SENSITIVE (glade_main_window, "local_file", FALSE);
}
