/* 
 * Copyright (C) 2002, 2003 Emmanuel Saracco <esaracco@noos.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA. 
 */

#include "bookmarks.h"


static GList *uc_bookmarks_get_links (void);


void
uc_bookmarks_save_changes (void)
{
  gchar *new_path = NULL;

  new_path = g_strconcat (uc_project_get_bookmarks_file (), ".uc", NULL);

  if (!rename (uc_project_get_bookmarks_file (), new_path))
    {
      xmlSaveFile (uc_project_get_bookmarks_file (), doc);
      uc_project_set_save_bookmarks (FALSE);
    }
  else
    uc_application_dialog_show (_("A error occured saving bookmarks file.\n"
				  "Nothing has been done."),
				GTK_MESSAGE_ERROR);

  g_free (new_path);
}


/*
 * remove a node on the treeview list.
 * -> the removal is virtual (to_delete = TRUE)
 */
void
uc_bookmarks_delete_link (const guint id)
{
  UCLinkProperties *prop = NULL;
  xmlNodePtr xml_node = NULL;
  GtkTreeIter iter;

  uc_project_set_save_bookmarks (TRUE);

  uc_application_treeview_get_selected_iter (treeview, &iter);
  prop = uc_lists_checked_links_lookup_by_uid (id);
  prop->to_delete = TRUE;

  /* !! HACK !!: don't know how to remove properly a node 
   * with libxml... any help? */
  xml_node = prop->xml_node;
  xml_node->prev->next = (xml_node->next) ? xml_node->next : NULL;
  /* !! HACK !! */

  gtk_tree_store_remove (treestore, &iter);
}


/*
 * get all the links of a given bookmarks
 * file
 */
static GList *
uc_bookmarks_get_links (void)
{
  GList *list = NULL;

  list = uc_xml_parser_get_tags (uc_project_get_bookmarks_file ());

  /* FIXME */
//  xmlFreeDoc (doc);

  return list;
}


/*
 * begin the bookmarks check
 */
void
uc_bookmarks_begin_check (void)
{
  time_t t;
  guint32 timer_id = 0;
  guint i_iter = 0;
  gchar *tmp = NULL;
  GtkWidget *widget = NULL;
  GtkTreeIter iter[UC_MAX_DEPTH_LEVEL];

  uc_application_draw_main_frames ();
  uc_application_update_ui ();

  UC_GET_WIDGET ("urls_list", main_window, widget);
  treeview = (GtkTreeView *) widget;

  uc_application_progress_dialog_show ();
  uc_application_update_ui ();

  uc_check_reset ();
  uc_project_free (UC_PROJECT_TYPE_BOOKMARKS);
  tmp = g_strconcat ("gURLChecker v", VERSION, " - ",
		     uc_project_get_bookmarks_file (), NULL);
  gtk_window_set_title (GTK_WINDOW (main_window), tmp);
  g_free (tmp);

  time (&t);
  timer_id = gtk_timeout_add (1000, &uc_report_timer_callback, (gpointer) t);

  uc_project_set_check_is_current (TRUE);
  uc_project_set_current_host (g_strdup (""));
  uc_project_set_current_port (g_strdup ("80"));
  uc_lists_checked_links_set (uc_bookmarks_get_links ());
  uc_project_set_check_is_current (FALSE);

  gtk_timeout_remove (timer_id);

  i_iter = 0;
  treestore = gtk_tree_store_new (N_COLUMNS,
				  G_TYPE_INT,
				  GDK_TYPE_PIXBUF,
				  GDK_TYPE_PIXBUF,
				  G_TYPE_STRING,
				  G_TYPE_STRING, GDK_TYPE_COLOR);

  uc_check_display_list (uc_lists_checked_links_get (), iter, i_iter, 0);

  uc_application_build_url_treeview ();

  gtk_widget_destroy (progress_dialog);

  /* activate some menu items */
  uc_application_menu_set_sensitive_all ("display", TRUE);
  uc_application_menu_set_sensitive_all ("search", TRUE);

  /* treeview is collapsed by default and no row is selected, so we deactivate 
     the "Collapse all the list" options */
  UC_WIDGET_SET_SENSITIVE (glade_main_window, "display_all", FALSE);
  UC_WIDGET_SET_SENSITIVE (glade_main_window, "display_collapse_all", FALSE);

  uc_check_cancel_set_value (FALSE);
}


/*
 * return TRUE if the given file is in xbel format
 */
gboolean
uc_bookmarks_format_is_xbel (const gchar * file)
{
  gchar *buffer = NULL;
  gboolean ret = FALSE;

  buffer = uc_utils_get_file_content (file);
  ret = (strstr (buffer, "<xbel ") != NULL);
  g_free (buffer);

  return ret;
}
