/* 
 * Copyright (C) 2002, 2003 Emmanuel Saracco <esaracco@noos.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA. 
 */

#ifndef H_UC_CHECK
#define H_UC_CHECK


#include "general.h"
#include "report.h"
#include "callbacks.h"
#include "cache.h"
#include "application.h"
#include "lists.h"
#include "html_parser.h"
#include "xml_parser.h"
#include "connection.h"
#include "project.h"


#define UC_CHECK_PARSER_SUSPEND_CONDITION \
	( \
		(uc_check_suspend_get_value ()) && \
		(!uc_check_cancel_get_value ()) \
	)

#define UC_CHECK_PARSER_WHILE_CONDITION \
	( \
		(item) && \
		(!uc_check_cancel_get_value ()) \
	)


#define UC_CHECK_ABORT_IF_CONDITION \
	uc_check_cancel_get_value () || \
	(!prop->url) || \
	g_strcasecmp (uc_project_get_current_host (), host) || \
	depth == uc_project_get_depth_level ()


/* definition of the treeview columns */
enum
{
  UID_COLUMN,
  LINK_STATUS_ICON_COLUMN,
  LINK_ICON_COLUMN,
  LABEL_COLUMN,
  URL_COLUMN,
  N_COLUMNS
};


/* struct to point on the current url or host elements
 * (for timeouts callback) */
struct _UCCurrentItem
{
  GList *list_ptr;
  UCLinkProperties *parent;
  xmlNodePtr xml_node;
  gchar *url;
  gchar *label;
}
currentItem;


/*
 * enum for return code of the href properties
 * function
 */
typedef enum
{
  UC_HTTP_CHECK_RETURN_OK,
  UC_HTTP_CHECK_RETURN_BAD,
  UC_HTTP_CHECK_RETURN_REDIR,
  UC_HTTP_CHECK_RETURN_IGNORE,
  UC_HTTP_CHECK_RETURN_FAKED_URL
}
UCHTTPCheckReturn;


void
uc_check_currentitem_init (gchar * url, gchar * label,
			   const xmlNodePtr * node);
void uc_check_timeout_links_free (void);
void uc_check_register_timeout_links (GList ** list);
void uc_check_wait (void);
void uc_check_insert_malformed_link (const UCHTMLTag * tag, GList * list);
gboolean uc_check_link_already_checked_with_insert (UCLinkProperties * prop,
						    gchar * url);
GList *uc_check_register_link (const gchar * normalized_url,
			       UCLinkProperties * lp);
UCLinkProperties *uc_check_link_get_properties (const guint depth,
						gchar * current_host,
						const gchar * current_path,
						UCHTMLTag * tag,
						gboolean * accept);
UCLinkProperties *uc_check_link_properties_node_new (void);
void uc_check_gdk_input_timeout_increment (void);
gboolean uc_check_gdk_input_timeout_locked (void);
void uc_check_mx_is_valid (void);
gboolean uc_check_status_is_ignored (const gchar * status);
gboolean uc_check_status_is_malformed (const gchar * status);
gboolean uc_check_status_is_good (const gchar * status);
gboolean uc_check_status_is_bad (const gchar * status);
gboolean uc_check_status_is_timeout (const gchar * status);
gboolean uc_check_status_is_email (const gchar * status);
void uc_check_suspend_continue (void);
gboolean uc_check_cancel_get_value (void);
gboolean uc_check_suspend_get_value (void);
gboolean uc_check_ignore_item_get_value (void);
void uc_check_timeout_set_value (const gboolean flag);
gboolean uc_check_timeout_get_value (void);
void uc_check_timeout_add ();
gint uc_check_treeview_get_selected_row_id (void);
void uc_check_alarm_add (const guint seconds, gpointer callback);
void uc_check_alarm_remove (void);
void uc_check_web_begin (void);
void uc_check_bookmarks_begin (void);
void uc_check_cancel_set_value (const gboolean value);
void uc_check_ignore_item_set_value (const gboolean value);
void uc_check_display_items_active_all (void);
void uc_check_display_list (GList * list, GtkTreeIter iter[], guint i_iter,
			    gchar * path);
void uc_check_display_list_with_only (GList * list, GtkTreeIter iter,
				      const guint view_type);
void uc_check_reset (void);
void uc_check_alarm_callback (int signo);
void uc_check_link_view_source (void);

#endif
