/*
 * Copyright (C) 2002, 2003 Emmanuel Saracco <esaracco@noos.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "connection.h"


static gchar *current_host = NULL;
static guint current_port = 0;
static guint fd = 0;
static gboolean uc_thread_exit = FALSE;


static guint uc_connection_socket_open (gchar * host, const guint port,
					gboolean force_args_use);
static gpointer uc_connection_socket_open_thread (gpointer fd);


/*
 * open a socket on the given host and port
 */
static guint
uc_connection_socket_open (gchar * host, const guint port,
			   gboolean force_args_use)
{
  struct sockaddr_in server_sockaddr;
  struct hostent *serverHostEnt = NULL;
  gchar buffer[BUFFER_URL_LEN + 1] = { 0 };
  gchar *host_real = NULL;
  guint port_real = 0;
  guint sock = 0;
  in_addr_t hostAddr = 0;
  gboolean ok = FALSE;
  gchar *proxy_host = NULL;
  guint proxy_port = 0;
  guint ret = 0;

  sock = socket (AF_INET, SOCK_STREAM, 0);

  if (sock <= 0)
    return ret;

  proxy_host = uc_project_get_proxy_host ();
  proxy_port = uc_project_get_proxy_port ();

  /* no proxy use for localhost */
  if ((!strcasecmp (host, "localhost")) || (strstr (host, "127.0.0.")))
    force_args_use = UC_SOCKET_OPEN_FORCE_ARGS_USE;

  if ((force_args_use == UC_SOCKET_OPEN_FORCE_ARGS_USE) ||
      (!strlen (proxy_host)))
    {
      host_real = host;
      port_real = port;
    }
  else
    {
      host_real = proxy_host;
      if (strlen (host_real))
	port_real = proxy_port;
      else
	{
	  host_real = host;
	  port_real = port;
	}
    }

  memset (&server_sockaddr, 0, sizeof (server_sockaddr));
  server_sockaddr.sin_family = AF_INET;
  server_sockaddr.sin_port = htons (port_real);

  uc_check_alarm_add (uc_project_get_check_timeout (),
		      &uc_check_alarm_callback);

  if ((serverHostEnt = gethostbyname (host_real)) != NULL)
    {
      inet_ntop (AF_INET,
		 (struct in_addr *) serverHostEnt->h_addr_list[0],
		 buffer, BUFFER_URL_LEN);
      inet_pton (AF_INET, buffer, &server_sockaddr.sin_addr);
    }
  else if (((in_addr_t) hostAddr = inet_addr (host_real)) != (in_addr_t) - 1)
    {
      memcpy (&server_sockaddr.sin_addr, &hostAddr, sizeof (hostAddr));
      serverHostEnt = gethostbyaddr (&hostAddr, sizeof (hostAddr), AF_INET);
    }

  if (serverHostEnt)
    ok = (!connect (sock, (struct sockaddr *) &server_sockaddr,
		    sizeof (server_sockaddr)));

  uc_check_alarm_remove ();

  if (!ok)
    close (sock), ret = 0;
  else
    ret = sock;

  return ret;
}


/*
 * threaded method, to attempt host connection
 * without freezing the ui
 */
static gpointer
uc_connection_socket_open_thread (gpointer data)
{
  fd = 0;
  fd = uc_connection_socket_open (current_host, current_port,
				  UC_SOCKET_OPEN_DEFAULT);

  uc_thread_exit = TRUE;
  g_thread_exit (NULL);

  return NULL;
}


/*
 * return a socket from the connection cache or
 * after creating it
 */
guint
uc_connection_open (gchar * host, const guint port)
{
  GThread *tid = NULL;
  GError *error = NULL;

  current_host = host;
  current_port = port;

  /* run connection attempt in a
   * thread to not hang the ui */
  uc_thread_exit = FALSE;
  if (!
      (tid =
       g_thread_create (uc_connection_socket_open_thread, NULL, FALSE,
			&error)))
    {
      g_warning ("can not create thread for connection...");
      return 0;
    }

  while (!uc_thread_exit)
    uc_application_update_ui ();

  /* connection failed */
  if (!fd)
    return 0;

  return fd;
}
