/*
 * Copyright (C) 2002, 2003 Emmanuel Saracco <esaracco@noos.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef H_UC_GENERAL
#define H_UC_GENERAL

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include "getopt.h"
#include <ctype.h>
#include <gnome.h>
#include <math.h>
#include <glade/glade.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <pthread.h>
#include <resolv.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

#ifdef __linux__
#define UC_SYSTEM_NAME "Linux"
#include <sys/vfs.h>
#endif

#ifdef __FreeBSD__
#define UC_SYSTEM_NAME "FreeBSD"
#include <sys/param.h>
#include <sys/mount.h>
#include <arpa/nameser.h>
#include <arpa/nameser_compat.h>
#endif

#include <signal.h>
#include <setjmp.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>

#define ELEMENTS(x) \
	(sizeof (x) / sizeof (x[0]))

#define PROPERTIES_FILENAME \
	"/gurlchecker-properties"

#define GLADE_XML_FILENAME \
	PACKAGE_DATA_DIR "/" PACKAGE "/ui/gurlchecker.glade"

#define UC_PIXMAPS_DIR \
	PACKAGE_DATA_DIR "/" PACKAGE "/ui/"

#define DIALOG_SETTINGS_DOCUMENT_TYPES_MAX_COLS \
	6

#define CHECK_BUTTON_MAX_DISPLAY \
	100

#define CHECK_MESSAGE_LABEL_FIRST \
	0
#define CHECK_MESSAGE_LABEL_SECOND \
	1
#define CHECK_MESSAGE_LABEL_THIRD \
	2

#define UC_CHECK_DISPLAY_GOODLINKS \
	0
#define UC_CHECK_DISPLAY_BADLINKS \
	1
#define UC_CHECK_DISPLAY_MALFORMED \
	2
#define UC_CHECK_DISPLAY_TIMEOUTS \
	10

#define STATUS_CODE_PAGE_OK \
	"200"
#define STATUS_CODE_NO_DATA \
	"204"
#define STATUS_CODE_MOVED_PERMANENTLY \
	"301"
#define STATUS_CODE_REDIRECTED \
	"302"
#define STATUS_CODE_RESTRICTED \
	"401"
#define STATUS_CODE_PAGE_NOT_FOUND \
	"404"
#define STATUS_CODE_TIMEOUT \
	"408"
#define STATUS_CODE_HOST_IS_UNREACHABLE \
	"999"
#define STATUS_CODE_UNKNOWN \
	"998"
#define STATUS_CODE_IGNORED_LINK \
	"997"
#define STATUS_CODE_MALFORMED_LINK \
	"899"
#define STATUS_CODE_FILE_PROTO_ERROR \
	"898"
#define STATUS_CODE_BAD_EMAIL_LINK \
	"799"
#define STATUS_CODE_BAD_EMAIL_LINK_MX \
	"798"
#define STATUS_CODE_GOOD_EMAIL_LINK \
	"797"

#define STATUS_CODE_PROPERTIES_CODE \
	0
#define STATUS_CODE_PROPERTIES_MESSAGE \
	1
#define STATUS_CODE_PROPERTIES_DESCRIPTION \
	2
#define STATUS_CODE_PROPERTIES_BACKGROUND \
	3

#define IGNORED_LINK_BGCOLOR \
	"pink"
#define BAD_LINK_BGCOLOR \
	"red"
#define GOOD_LINK_BGCOLOR \
	"green"
#define TIMEOUT_LINK_BGCOLOR \
	"purple"
#define HOST_UNREACHABLE_LINK_BGCOLOR \
	"yellow"
#define NO_DATA_LINK_BGCOLOR \
	"orange"
#define RESTRICTED_LINK_BGCOLOR \
	"blue"
#define MALFORMED_LINK_BGCOLOR \
	"brown"
#define FILE_ERROR_LINK_BGCOLOR \
	"silver"
#define BAD_EMAIL_LINK_BGCOLOR \
	"red"
#define GOOD_EMAIL_LINK_BGCOLOR \
	"green"

#define UC_CHECK_PROTOCOLS_ACCEPTED \
	"HTTP"
#define UC_CHECK_PROTOCOLS_ACCEPTED_LEN \
	1
#define UC_CHECK_PROTOCOLS_GET_ACCEPTED \
        g_strsplit( \
                        UC_CHECK_PROTOCOLS_ACCEPTED, \
                        UC_SPLIT_DELIMITER, \
                        UC_CHECK_PROTOCOLS_ACCEPTED_LEN \
                  )

#define UC_SPLIT_DELIMITER \
	","
#define UC_CONFIG_PROPERTIES_DEFAULT_CONTENT_TYPES_DOCUMENTS \
	"mov,cfm,html,htm,pl,phtml,pdf,doc,shtml,asp,php,xml,xls,swf,sxi," \
	"tar,gz,tgz,zip,bz2,deb,rpm"
#define UC_CONFIG_PROPERTIES_DEFAULT_CONTENT_TYPES_DOCUMENTS_LEN \
	22
#define UC_CONFIG_PROPERTIES_DEFAULT_CONTENT_TYPES_IMAGES \
	"png,gif,jpeg,xpm,bmp,jpg"
#define UC_CONFIG_PROPERTIES_DEFAULT_CONTENT_TYPES_IMAGES_LEN \
	6
#define UC_CONFIG_PROPERTIES_GET_DEFAULT_CONTENT_TYPES_DOCUMENTS \
	g_strsplit( \
		UC_CONFIG_PROPERTIES_DEFAULT_CONTENT_TYPES_DOCUMENTS, \
		UC_SPLIT_DELIMITER, \
		UC_CONFIG_PROPERTIES_DEFAULT_CONTENT_TYPES_DOCUMENTS_LEN \
        )
#define UC_CONFIG_PROPERTIES_GET_DEFAULT_CONTENT_TYPES_IMAGES \
	g_strsplit( \
		UC_CONFIG_PROPERTIES_DEFAULT_CONTENT_TYPES_IMAGES, \
		UC_SPLIT_DELIMITER, \
		UC_CONFIG_PROPERTIES_DEFAULT_CONTENT_TYPES_IMAGES_LEN \
        )

#define UC_CONFIG_PROPERTIES_GENERAL_DEFAULT_WORKING_PATH \
	"/tmp/"
#define UC_CONFIG_PROPERTIES_GENERAL_DEFAULT_WEB_BROWSER_PATH \
	"/usr/bin/mozilla"
#define UC_CONFIG_PROPERTIES_URL_CONTENT_TYPES_DOCUMENTS \
	"url/content-types/documents"
#define UC_CONFIG_PROPERTIES_URL_CONTENT_TYPES_IMAGES \
	"url/content-types/images"
#define UC_CONFIG_PROPERTIES_URL_NETWORKING_DEPTH_LEVEL \
	"url/networking/depth-level"
#define UC_CONFIG_PROPERTIES_URL_NETWORKING_CHECK_WAIT \
	"url/networking/check-wait"
#define UC_CONFIG_PROPERTIES_FILTERS_RETRIEVE_IMAGES_CONTENT \
	"filters/content-management/retrieve-images-source"
#define UC_CONFIG_PROPERTIES_FILTERS_RETRIEVE_ARCHIVES_CONTENT \
	"filters/content-management/retrieve-archives-source"
#define UC_CONFIG_PROPERTIES_FILTERS_CHECK_HTML_COMMENTS \
	"filters/content-management/check-HTML-content"
#define UC_CONFIG_PROPERTIES_FILTERS_REJECT_DIRECTORIES \
	"filters/reject/directories"
#define UC_CONFIG_PROPERTIES_FILTERS_REJECT_DOMAINS \
	"filters/reject/domains"
#define UC_CONFIG_PROPERTIES_FILTERS_PROTOCOLS_FILE_ERROR \
	"filters/protocols/file_is_error"
#define UC_CONFIG_PROPERTIES_FILTERS_PROTOCOLS_FILE \
	"filters/protocols/file"
#define UC_CONFIG_PROPERTIES_FILTERS_PROTOCOLS_MAILTO \
	"filters/protocols/mailto"
#define UC_CONFIG_PROPERTIES_FILTERS_PROTOCOLS_MAILTO_CHECK_MX \
	"filters/protocols/mailto_check_mx"
#define UC_CONFIG_PROPERTIES_GENERAL_NETWORKING_TIMEOUT \
	"general/networking/timeout"
#define UC_CONFIG_PROPERTIES_GENERAL_WEB_BROWSER_PATH \
	"general/networking/web-browser"
#define UC_CONFIG_PROPERTIES_GENERAL_NETWORKING_PROXY_CONFIG \
	"general/networking/proxy-config"
#define UC_CONFIG_PROPERTIES_GENERAL_NETWORKING_PROXY_HOST \
	"general/networking/proxy-host"
#define UC_CONFIG_PROPERTIES_GENERAL_NETWORKING_PROXY_PORT \
	"general/networking/proxy-port"
#define UC_CONFIG_PROPERTIES_GENERAL_NETWORKING_PROXY_LOGIN \
	"general/networking/proxy-login"
#define UC_CONFIG_PROPERTIES_GENERAL_NETWORKING_PROXY_PASSWORD \
	"general/networking/proxy-password"
#define UC_CONFIG_PROPERTIES_GENERAL_WORKING_PATH \
	"general/directories/working-path"
#define UC_CONFIG_PROPERTIES_GENERAL_DEBUG \
	"general/debug/active"
#define UC_CONFIG_PROPERTIES_GENERAL_DEBUG_DUMP_PROPERTIES \
	"general/debug/dump-properties"

#define UC_CONFIG_GET_VECTOR(key, vector, size) \
  gnome_config_get_vector (PROPERTIES_FILENAME "/" key, size, vector)
#define UC_CONFIG_GET_STRING(key) \
        gnome_config_get_string (PROPERTIES_FILENAME "/" key)
#define UC_CONFIG_GET_BOOL(key) \
        gnome_config_get_bool (PROPERTIES_FILENAME "/" key)
#define UC_CONFIG_GET_INT(key) \
        gnome_config_get_int (PROPERTIES_FILENAME "/" key)

#define UC_CONFIG_SET_STRING(key, value) \
        gnome_config_set_string (PROPERTIES_FILENAME "/" key, value)
#define UC_CONFIG_SET_INT(key, value) \
        gnome_config_set_int (PROPERTIES_FILENAME "/"  key, value)
#define UC_CONFIG_SET_VECTOR(key, values, size) \
        gnome_config_set_vector (g_strdup_printf("%s/%s", \
				PROPERTIES_FILENAME, key), size, values)

#define UC_WIDGET_SET_SENSITIVE(glade_pointer, name, value) \
         gtk_widget_set_sensitive ( \
           glade_xml_get_widget((GladeXML *) glade_pointer, name), \
           value)
#define UC_GET_WIDGET(name, window, widget) \
	 widget = (GtkWidget *) g_object_get_data (G_OBJECT (window), name)
#define UC_SET_WIDGET(name, window, widget) \
	g_object_set_data_full (G_OBJECT (window), name, \
	gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)
#define UC_GLADE_INIT(name, glade, widget) \
	 glade = glade_xml_new (GLADE_XML_FILENAME, name, NULL); \
	 glade_xml_signal_autoconnect (glade); \
	 widget = glade_xml_get_widget (glade, name);
#define UC_GLADE_GET_WIDGET(name, glade, widget) \
	widget =  glade_xml_get_widget (glade, name);
#define UC_LABEL_DISPLAY_IN_TABLE(table, label, name, p1, p2, p3, p4) \
        gtk_label_set_markup (GTK_LABEL (label), name); \
        gtk_table_attach (GTK_TABLE (table), label, p1, p2, p3, p4, \
		(GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0)

#define LIST_LABEL_MAX_LEN \
	40
#define BUFFER_LEN \
	2048
#define BUFFER_URL_LEN \
	2048
#define TAG_LEN \
	255
#define URL_PORT_MAX_LEN \
	5
#define HEADER_STATUS_CODE_LEN \
	3
#define HEADER_STATUS_CODE_FIRST_POS \
	9

#define MAX_URLS_CHECK \
	UC_CONFIG_GET_INT (UC_CONFIG_PROPERTIES_URL_NETWORKING_CHECK_LEVEL)
#define MAX_CHECK_DEPTH \
	UC_CONFIG_GET_INT (UC_CONFIG_PROPERTIES_URL_NETWORKING_DEPTH_LEVEL)
#define CHECK_WAIT \
	UC_CONFIG_GET_INT (UC_CONFIG_PROPERTIES_URL_NETWORKING_CHECK_WAIT)

#define PROTOCOL_HTTP \
	"http"
#define CONTENT_TYPE \
	"Content-Type"
#define CONTENT_LENGTH \
	"Content-Length"
#define CONTENT_TYPE_HTML \
	"text/html"

#define URL_DEFAULT_PORT \
	"80"

#define UC_SOCKET_OPEN_FORCE_ARGS_USE \
	TRUE
#define UC_SOCKET_OPEN_DEFAULT \
	FALSE

/* security adding for free partition space verification for
 * the working directory */
#define UC_SECURE_BYTES_ADDING \
	5000000
#define UC_CHECK_MINIMUM_SPACE_REQUIRED \
	10000000

#define CHECK_WAIT_DEFAULT \
	0
#define CHECK_TIMEOUT_DEFAULT \
	60
#define CHECK_LEVEL_DEFAULT \
	1000
#define DEPTH_LEVEL_DEFAULT \
	10
#define UC_MAX_DEPTH_LEVEL \
	100
#define PROXY_PORT_DEFAULT \
	8080

#define USER_AGENT \
	"Mozilla/5.0 gURLChecker/" VERSION " (" UC_SYSTEM_NAME ")"
#define REFERER \
	"http://savannah.nongnu.org/projects/gurlchecker/"


xmlDocPtr doc;

/* global pointers for glade xml file */
GladeXML *glade_main_window;
GladeXML *glade_settings_dialog;
GladeXML *glade_new_project_dialog;
GladeXML *glade_progress_dialog;
GladeXML *glade_view_source_dialog;
GladeXML *glade_view_image_dialog;
GladeXML *glade_similar_links_dialog;
GladeXML *glade_page_information_dialog;
GladeXML *glade_add_filter_directory_dialog;
GladeXML *glade_add_filter_domain_dialog;
GladeXML *glade_new_search_dialog;
GladeXML *glade_search_dialog;
GladeXML *glade_cancel_dialog;
GladeXML *glade_save_bookmarks_dialog;


/* global windows and dialogs pointers */
GtkWidget *main_window;
GtkWidget *new_project_dialog;
GtkWidget *settings_dialog;
GtkWidget *progress_dialog;
GtkWidget *view_source_dialog;
GtkWidget *view_image_dialog;
GtkWidget *similar_links_dialog;
GtkWidget *page_information_dialog;
GtkWidget *add_filter_directory_dialog;
GtkWidget *add_filter_domain_dialog;
GtkWidget *new_search_dialog;
GtkWidget *search_dialog;
GtkWidget *cancel_dialog;
GtkWidget *save_bookmarks_dialog;


/* treeview pointers */
GtkTreeView *treeview;		// url list
GtkTreeView *treeview_filter_directories;	// rejected directories
GtkTreeView *treeview_filter_domains;	// rejected domains
GtkTreeView *treeview_search;	// result of the search


/* treestore for the url treeview list */
GtkTreeStore *treestore;
GtkListStore *treestore_filter_directories;	// rejected directories
GtkListStore *treestore_filter_domains;	// rejected domains

GtkListStore *treestore_search;	// result of the search

gboolean uc_refresh_page;


/*
 * structure to store search properties
 */
struct UCSearchState
{
  gchar *string;
  gboolean case_sensitive;
  gboolean pages_content;
  gboolean links_name;
  gboolean meta_content;
  gboolean emails;
}
searchState;


/* structure to store status code properties */
typedef struct
{
  gchar *message;
  gchar *description;
  GdkColor background;
  GdkColor foreground;
  GdkPixbuf *icon_file;
}
UCStatusCode;


/* enum to identify links types */
typedef enum
{
  LINK_TYPE_NONE,
  LINK_TYPE_HREF,
  LINK_TYPE_EMAIL,
  LINK_TYPE_FILE_HREF,
  LINK_TYPE_FILE_IMAGE,
  LINK_TYPE_IMAGE,
  LINK_TYPE_META,
  LINK_TYPE_FRAME,
  LINK_TYPE_BOOKMARK_FOLDER,
  LINK_TYPES_LEN
}
UCLinkType;


/*
 * errors type
 */
typedef enum
{
  UC_ERROR_MALFORMED_LINK,
  UC_ERROR_NONE
}
UCError;


/* status for email links */
typedef enum
{
  UC_EMAIL_SYNTAX_BAD,
  UC_EMAIL_SYNTAX_MX_BAD,
  UC_EMAIL_OK
}
UCEmailStatus;


typedef struct
{
  guint depth;
  gchar *name;
  gchar *label;
  guchar *value;
  GHashTable *attr;
  UCLinkType type;
  gboolean malformed;
}
UCHTMLTag;


/* structure to store links properties */
typedef struct _UCLinkProperties
{
  guint uid;
  gchar *treeview_path;
  struct _UCLinkProperties *parent;
  UCLinkType link_type;
  GdkPixbuf *link_icon;
  GdkPixbuf *status_icon;
  gchar *link_value;
  gchar *url;
  gchar *label;
  gchar *protocol;
  gchar *h_name;
  gchar *port;
  gchar *path;
  gchar *file;
  gchar *args;
  gchar *domain;
  GHashTable *header;
  guint header_size;
  guint depth_level;
  gboolean is_parsable;
  gboolean is_downloadable;
  gboolean checked;
  xmlNodePtr xml_node;
  gboolean to_delete;
  GList *metas;
  GList *emails;
  GList *all_links;
  GList *childs;
  GList *similar_links_parents;
  GList *tags;
}
UCLinkProperties;


/* taken from glibc ghash.c to avoid use of
 * g_hash_table_foreach function */
typedef struct _GHashNode GHashNode;
struct _GHashNode
{
  gpointer key;
  gpointer value;
  GHashNode *next;
};
struct _GHashTable
{
  gint size;
  gint nnodes;
  GHashNode **nodes;
  GHashFunc hash_func;
  GEqualFunc key_equal_func;
  GDestroyNotify key_destroy_func;
  GDestroyNotify value_destroy_func;
};
/*********** end of glibc code ***********/
#endif
