/*
 * Copyright (C) 2002, 2003 Emmanuel Saracco <esaracco@noos.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "project.h"

struct
{
  G_CONST_RETURN gchar *local_charset;
  gboolean check_is_current;
  gboolean save_bookmarks;
  gchar *url;
  gchar *bookmarks_file;
  gchar *current_host;
  gchar *current_port;
  guint check_wait;
  guint depth_level;
  gchar *working_path;
  gchar *cache_name;
  guint proto_file_is_error;
  guint proto_file_check;
  guint proto_mailto;
  guint proto_mailto_check_mx;
  guint check_chroot;
  guint limit_local;
  guint use_proxy;
  gchar *proxy_host;
  guint proxy_port;
  gchar *chroot_path;
  guint images_content;
  guint archives_content;
  guint check_comments;
  guint check_timeout;
  guint debug_mode;
  guint dump_properties;
}
static project;


void
uc_project_new (void)
{
  gchar *host = NULL;
  guint port = 0;
  G_CONST_RETURN gchar *charset;

  g_get_charset (&charset);
  uc_project_set_local_charset (charset);

  uc_project_set_cache_name (g_strdup_printf ("%s_%d", PACKAGE, getpid ()));

  uc_project_set_depth_level (UC_CONFIG_GET_INT
			      (UC_CONFIG_PROPERTIES_URL_NETWORKING_DEPTH_LEVEL));

  uc_project_set_check_wait (UC_CONFIG_GET_INT
			     (UC_CONFIG_PROPERTIES_URL_NETWORKING_CHECK_WAIT));

  uc_project_set_working_path (UC_CONFIG_GET_STRING
			       (UC_CONFIG_PROPERTIES_GENERAL_WORKING_PATH));

  uc_project_set_proto_file_is_error (UC_CONFIG_GET_INT
				      (UC_CONFIG_PROPERTIES_FILTERS_PROTOCOLS_FILE_ERROR));

  uc_project_set_proto_file_check (UC_CONFIG_GET_INT
				   (UC_CONFIG_PROPERTIES_FILTERS_PROTOCOLS_FILE));

  uc_project_set_proto_mailto (UC_CONFIG_GET_INT
			       (UC_CONFIG_PROPERTIES_FILTERS_PROTOCOLS_MAILTO));

  uc_project_set_proto_mailto_check_mx (UC_CONFIG_GET_INT
					(UC_CONFIG_PROPERTIES_FILTERS_PROTOCOLS_MAILTO_CHECK_MX));

  uc_project_set_download_images_content (UC_CONFIG_GET_INT
					  (UC_CONFIG_PROPERTIES_FILTERS_RETRIEVE_IMAGES_CONTENT));

  uc_project_set_download_archives_content (UC_CONFIG_GET_INT
					    (UC_CONFIG_PROPERTIES_FILTERS_RETRIEVE_ARCHIVES_CONTENT));

  uc_project_set_check_comments (UC_CONFIG_GET_INT
				 (UC_CONFIG_PROPERTIES_FILTERS_CHECK_HTML_COMMENTS));

  uc_project_set_check_timeout (UC_CONFIG_GET_INT
				(UC_CONFIG_PROPERTIES_GENERAL_NETWORKING_TIMEOUT));

  uc_project_set_debug_mode (UC_CONFIG_GET_INT
			     (UC_CONFIG_PROPERTIES_GENERAL_DEBUG));

  uc_project_set_dump_properties (UC_CONFIG_GET_INT
				  (UC_CONFIG_PROPERTIES_GENERAL_DEBUG_DUMP_PROPERTIES));

  uc_utils_get_gnome_proxy_conf (&host, &port);
  uc_project_set_use_proxy ((strlen (host) > 0));
  if (uc_project_get_use_proxy ())
    {
      uc_project_set_proxy_host (host);
      uc_project_set_proxy_port (port);
    }
  else
    {
      uc_project_set_proxy_host (g_strdup (""));
      uc_project_set_proxy_port (0);
    }

  uc_project_set_save_bookmarks (FALSE);
}


void
uc_project_free (UCProjectType type)
{
  if (type == UC_PROJECT_TYPE_BOOKMARKS || type == UC_PROJECT_TYPE_LOCAL_FILE)
    {
      g_free (project.url), project.url = NULL;
      g_free (project.current_host), project.current_host = NULL;
      g_free (project.current_port), project.current_port = NULL;
    }
  else if (type == UC_PROJECT_TYPE_WEB_SITE)
    g_free (project.bookmarks_file), project.bookmarks_file = NULL;
  else
    {
      g_free (project.url), project.url = NULL;
      g_free (project.bookmarks_file), project.bookmarks_file = NULL;
      g_free (project.current_host), project.current_host = NULL;
      g_free (project.current_port), project.current_port = NULL;
      g_free (project.working_path), project.working_path = NULL;
      g_free (project.cache_name), project.cache_name = NULL;
      g_free (project.proxy_host), project.proxy_host = NULL;
      project.local_charset = NULL;
    }

  uc_project_set_save_bookmarks (FALSE);
  project.chroot_path = NULL;

}


gboolean
uc_project_get_save_bookmarks (void)
{
  return project.save_bookmarks;
}


G_CONST_RETURN gchar *
uc_project_get_local_charset (void)
{
  return project.local_charset;
}


gboolean
uc_project_get_check_is_current (void)
{
  return project.check_is_current;
}


gchar *
uc_project_get_url (void)
{
  return project.url;
}


gchar *
uc_project_get_bookmarks_file (void)
{
  return project.bookmarks_file;
}


gchar *
uc_project_get_current_host (void)
{
  return project.current_host;
}


gchar *
uc_project_get_current_port (void)
{
  return project.current_port;
}


guint
uc_project_get_check_wait (void)
{
  return project.check_wait;
}


guint
uc_project_get_depth_level (void)
{
  return project.depth_level;
}


gchar *
uc_project_get_working_path (void)
{
  return project.working_path;
}


gchar *
uc_project_get_cache_name (void)
{
  return project.cache_name;
}


guint
uc_project_get_proto_file_is_error (void)
{
  return project.proto_file_is_error;
}


guint
uc_project_get_proto_file_check (void)
{
  return project.proto_file_check;
}


guint
uc_project_get_proto_mailto (void)
{
  return project.proto_mailto;
}


guint
uc_project_get_proto_mailto_check_mx (void)
{
  return project.proto_mailto_check_mx;
}


guint
uc_project_get_check_chroot (void)
{
  return project.check_chroot;
}


guint
uc_project_get_limit_local (void)
{
  return project.limit_local;
}


guint
uc_project_get_use_proxy (void)
{
  return project.use_proxy;
}


gchar *
uc_project_get_proxy_host (void)
{
  return project.proxy_host;
}


guint
uc_project_get_proxy_port (void)
{
  return project.proxy_port;
}


gchar *
uc_project_get_chroot_path (void)
{
  return project.chroot_path;
}


guint
uc_project_get_download_images_content (void)
{
  return project.images_content;
}


guint
uc_project_get_download_archives_content (void)
{
  return project.archives_content;
}


guint
uc_project_get_check_comments (void)
{
  return project.check_comments;
}


guint
uc_project_get_check_timeout (void)
{
  return project.check_timeout;
}


guint
uc_project_get_debug_mode (void)
{
  return project.debug_mode;
}


guint
uc_project_get_dump_properties (void)
{
  return project.dump_properties;
}


void
uc_project_set_save_bookmarks (const gboolean value)
{
  project.save_bookmarks = value;
  UC_WIDGET_SET_SENSITIVE (glade_main_window, "save_bookmarks", value);
}


void
uc_project_set_local_charset (G_CONST_RETURN gchar * value)
{
  project.local_charset = value;
}


void
uc_project_set_check_is_current (gboolean value)
{
  project.check_is_current = value;
}


void
uc_project_set_url (gchar * value)
{
  g_free (project.url);
  project.url = value;
}


void
uc_project_set_bookmarks_file (gchar * value)
{
  g_free (project.bookmarks_file);
  project.bookmarks_file = value;
}


void
uc_project_set_current_host (gchar * value)
{
  g_free (project.current_host);
  project.current_host = value;
}


void
uc_project_set_current_port (gchar * value)
{
  g_free (project.current_port);
  project.current_port = value;
}


void
uc_project_set_check_wait (guint value)
{
  project.check_wait = value;
}


void
uc_project_set_depth_level (guint value)
{
  project.depth_level = value;
}


void
uc_project_set_working_path (gchar * value)
{
  project.working_path = value;
}


void
uc_project_set_cache_name (gchar * value)
{
  project.cache_name = value;
}


void
uc_project_set_proto_file_is_error (guint value)
{
  project.proto_file_is_error = value;
}


void
uc_project_set_proto_file_check (guint value)
{
  project.proto_file_check = value;
}


void
uc_project_set_proto_mailto (guint value)
{
  project.proto_mailto = value;
}


void
uc_project_set_proto_mailto_check_mx (guint value)
{
  project.proto_mailto_check_mx = value;
}


void
uc_project_set_check_chroot (guint value)
{
  project.check_chroot = value;
}


void
uc_project_set_limit_local (guint value)
{
  project.limit_local = value;
}


void
uc_project_set_use_proxy (guint value)
{
  project.use_proxy = value;
}


void
uc_project_set_proxy_host (gchar * value)
{
  project.proxy_host = value;
}


void
uc_project_set_proxy_port (guint value)
{
  project.proxy_port = value;
}


void
uc_project_set_chroot_path (gchar * value)
{
  project.chroot_path = value;
}


void
uc_project_set_download_images_content (guint value)
{
  project.images_content = value;
}


void
uc_project_set_download_archives_content (guint value)
{
  project.archives_content = value;
}


void
uc_project_set_check_comments (guint value)
{
  project.check_comments = value;
}


void
uc_project_set_check_timeout (guint value)
{
  project.check_timeout = value;
}


void
uc_project_set_debug_mode (guint value)
{
  project.debug_mode = value;
}


void
uc_project_set_dump_properties (guint value)
{
  project.dump_properties = value;
}
