/*
 * Copyright (C) 2002, 2003 Emmanuel Saracco <esaracco@noos.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef H_UC_PROJECT
#define H_UC_PROJECT

#include "general.h"
#include "utils.h"

/* enum of project types */
typedef enum
{
  UC_PROJECT_TYPE_WEB_SITE,
  UC_PROJECT_TYPE_LOCAL_FILE,
  UC_PROJECT_TYPE_BOOKMARKS,
  UC_PROJECT_TYPE_ALL
}
UCProjectType;


void uc_project_free (UCProjectType type);
void uc_project_new (void);
gboolean uc_project_get_save_bookmarks (void);
G_CONST_RETURN gchar *uc_project_get_local_charset (void);
gboolean uc_project_get_check_is_current (void);
gchar *uc_project_get_url (void);
gchar *uc_project_get_bookmarks_file (void);
gchar *uc_project_get_current_host (void);
gchar *uc_project_get_current_port (void);
gchar *uc_project_get_proxy_host (void);
gchar *uc_project_get_working_path (void);
gchar *uc_project_get_chroot_path (void);
gchar *uc_project_get_cache_name (void);
guint uc_project_get_proto_file_is_error (void);
guint uc_project_get_proto_file_check (void);
guint uc_project_get_proto_mailto (void);
guint uc_project_get_proto_mailto_check_mx (void);
guint uc_project_get_check_wait (void);
guint uc_project_get_depth_level (void);
guint uc_project_get_check_chroot (void);
guint uc_project_get_limit_local (void);
guint uc_project_get_use_proxy (void);
guint uc_project_get_proxy_port (void);
guint uc_project_get_download_images_content (void);
guint uc_project_get_download_archives_content (void);
guint uc_project_get_check_comments (void);
guint uc_project_get_check_timeout (void);
guint uc_project_get_debug_mode (void);
guint uc_project_get_dump_properties (void);
void uc_project_set_save_bookmarks (const gboolean value);
void uc_project_set_local_charset (G_CONST_RETURN gchar * value);
void uc_project_set_check_is_current (gboolean value);
void uc_project_set_url (gchar * value);
void uc_project_set_bookmarks_file (gchar * value);
void uc_project_set_current_host (gchar * value);
void uc_project_set_current_port (gchar * value);
void uc_project_set_check_wait (guint value);
void uc_project_set_depth_level (guint value);
void uc_project_set_working_path (gchar * value);
void uc_project_set_cache_name (gchar * value);
void uc_project_set_proto_file_is_error (guint value);
void uc_project_set_proto_file_check (guint value);
void uc_project_set_proto_mailto (guint value);
void uc_project_set_proto_mailto_check_mx (guint value);
void uc_project_set_check_chroot (guint value);
void uc_project_set_limit_local (guint value);
void uc_project_set_use_proxy (guint);
void uc_project_set_proxy_host (gchar * value);
void uc_project_set_proxy_port (guint value);
void uc_project_set_chroot_path (gchar * value);
void uc_project_set_download_images_content (guint value);
void uc_project_set_download_archives_content (guint value);
void uc_project_set_check_comments (guint value);
void uc_project_set_check_timeout (guint value);
void uc_project_set_debug_mode (guint value);
void uc_project_set_dump_properties (guint value);

#endif
