/*
 * Copyright (C) 2002, 2003 Emmanuel Saracco <esaracco@noos.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "timeouts.h"


/* global variable to know when operation take to much
 * time */
static guint uc_timeout_gdk_input_value = 0;

/* TRUE if we are processing data in a gdk_input
 * method */
static gboolean uc_timeout_gdk_input_lock_value = FALSE;

/* TRUE if a timeout occured */
static gboolean uc_timeout_value = FALSE;


/*
 * set a SIGALRM
 */
void
uc_timeout_alarm_add (const guint seconds, gpointer callback)
{
  struct sigaction action;

  uc_timeout_set (FALSE);
  sigemptyset (&(action.sa_mask));
  action.sa_flags = 0;
  action.sa_handler = callback;

  sigaction (SIGALRM, &action, NULL);
  alarm (seconds);
}


/*
 * remove a SIGALRM
 */
void
uc_timeout_alarm_remove (void)
{
  alarm (0);
}



/*
 * set the timeout flag
 */
void
uc_timeout_set (const gboolean flag)
{
  uc_timeout_value = flag;
}


/*
 * get the timeout flag
 */
gboolean
uc_timeout_get (void)
{
  return (uc_timeout_value || uc_timeout_gdk_input_get ());
}


/*
 * increment the internal timeout counter
 */
void
uc_timeout_gdk_input_increment (void)
{
  uc_timeout_gdk_input_value++;
}


/*
 * avoid timeout when manipulating data
 */
void
uc_timeout_gdk_input_lock (void)
{
  uc_timeout_gdk_input_lock_value = TRUE;
}


/*
 * authorize timeout
 */
void
uc_timeout_gdk_input_unlock (void)
{
  uc_timeout_gdk_input_lock_value = FALSE;
}


/*
 * return TRUE if we can increment timeout
 * value
 */
gboolean
uc_timeout_gdk_input_locked (void)
{
  return uc_timeout_gdk_input_lock_value;
}


/*
 * initialize the timeout flag for
 * gdk_input methods
 */
void
uc_timeout_gdk_input_init (void)
{
  uc_timeout_gdk_input_value = 0;
  uc_timeout_gdk_input_unlock ();
}


/*
 * TRUE if the waiting time has
 * been passed
 */
gboolean
uc_timeout_gdk_input_get (void)
{
  return (uc_timeout_gdk_input_value > uc_project_get_check_timeout () + 1);
}

/*
 * end the timeout for gdk input methods
 * -> if a timeout occured, set the main
 *  timeout flag to TRUE
 */
void
uc_timeout_gdk_input_end (void)
{
  if (uc_timeout_gdk_input_value >= uc_project_get_check_timeout ())
    uc_check_alarm_callback (0);

  uc_timeout_gdk_input_reset ();
}

/*
 * same as init () method
 */
void
uc_timeout_gdk_input_reset (void)
{
  uc_timeout_gdk_input_init ();
}
